<?php
  
// admingroups.php -  List, modify, and add groups from SIDB
// admingroups.php - author: SVI Group University Utrecht and Nico Stuurman
  /***************************************************************************
  * This script displays a table with groups in SIDB. It can only be called  *
  *  by the sysadmin.
  * Functions to add, modify and deleted groups are integrated.              *
  *
  * Modified by Frank on 28-06-2000. 
  *
  * Authors: SVI group University Utrecht and Nico Stuurman                  *
  * email: stuurman@rulbim.leidenuniv.nl                                     *
  * demo at: https://impi.leidenuniv.nl                                      *
  * Last modified 12/17/00 by Nico Stuurman                                  *
  * Copyright (c) 2000 by Nico Stuurman                                      *
  * ------------------------------------------------------------------------ *
  *  This program is free software; you can redistribute it and/or modify it *
  *  under the terms of the GNU General Public License as published by the   *
  *  Free Software Foundation; either version 2 of the License, or (at your  *
  *  option) any later version.                                              *
  \**************************************************************************/                                             

// main include thingies
require("include.php");

Auth::allowonly($G_SUPER);

// register variables
$PHP_SELF = $HTTP_SERVER_VARS ["PHP_SELF"];
$get_vars = "mod,groupid,groupname,add,";
globalize_vars ($get_vars,"HTTP_GET_VARS");
$post_vars = "groupid,groupname,submit,";
globalize_vars ($post_vars, "HTTP_POST_VARS");

// main global vars
$title = "Admin Groups";

////
// Adds group to database and dir with groupname in archive dir.
// An error string is returned when problems occur 
function add_new_group ($groupname) {
   global $archive_dir;

   // check if a groupname is entered
   if ($groupname) {
 
      $db_query = "SELECT * FROM groups WHERE groupname='$groupname'";
      $db_result = db_exec($db_query);
      $db_row = db_numrows($db_result);
 
      // test if a result is found
      if ($db_row > 0) {
          return "groupname already exists; please try again";
      }

      else {
         if ( do_database_query ("INSERT INTO groups(groupname) VALUES('$groupname')") ) {
            echo "Group <b>$groupname</b> added,";
 
            // add group directory to filesystem
            if ( !is_dir($archive_dir) || !is_writable($archive_dir) ) {
               echo "Error: main archive $archive_dir  dir not accessable.";
               exit;
            }
            mkdir("$archive_dir/$groupname", 0777);
            echo " dir <b>$groupname</b> added to filesystem.<br><br>";
         }
         else 
            echo "Group was not added<br>";
      }      
   }
   else
      return "Please enter a groupname!";
}

////
// Change that groupname.  On error, it returns an error string
function modify_group ($groupid, $new_name) {
   global $archive_dir;

   // Only change the name when there is a new one provided
   if ($new_name) {

      // check whether a group with the new name already exists
      $db_result = do_database_query ("SELECT groupid FROM groups WHERE 
                   (groupname='$new_name' AND NOT groupid='$groupid');");
      $db_row = db_numrows ($db_result);
      if ($db_row > 0)
         return "Groupname <b>$new_name</b> already exists, please select another name";     
      // when not, change the name
      else {
         // get the old name
         $old_name = get_group_info ($groupid,"groupname");
         $db_query = "UPDATE groups SET groupname='$new_name' WHERE groupid='$groupid';";
         if (do_database_query ($db_query))
            echo "New groupname is <b>$new_name</b><br><br>";
            echo "New group archive is <b>$archive_dir/$new_name</b><br><br>";
          
         // change group directory name
            if ( !is_dir($archive_dir) || !is_writable($archive_dir) ) {
               echo "Error: main archive $archive_dir  dir not accessable.";
               exit;
            } 
            if (@rename("$archive_dir/$old_name", "$archive_dir/$new_name"))
	    {
               echo " directory <b>$new_name</b> added to the filesystem.<br><br>";         
	    }
	    else
	    {
		echo "Error: group archive $archive_dir/$new_name  dir not accessable.";
		exit;
	    }

      }
   }
   else
      return "Please enter a groupname!";
}

////
// delete given group
function delete_group ($groupid) {
   global $archive_dir;

   // check if a groupname is entered
   if (!$groupid) 
      return false;
 
   $db_groupname_query = "SELECT groupname FROM groups WHERE groupid='$groupid'";
   $db_groupname_result = db_exec($db_groupname_query);
   $db_groupname_rows = db_numrows($db_groupname_result);
 
   // just in case, test if a result has been found
   if ($db_groupname_rows == 0)
      echo "Group does not exists; please try again";
   else {
 
      // get groupname from database result
      $db_groupname_row_result = db_fetch_row($db_groupname_result, 0);
      $groupname = $db_groupname_row_result[0];
 
      // test if group has no more users
      $db_users_query = "SELECT userid FROM users WHERE groupid='$groupid'";
      $db_users_result = db_exec($db_users_query);
      $db_users_rows = db_numrows($db_users_result);
      if ($db_users_rows > 0) {
         echo "Group <b>$groupname</b> can not be removed since it still has users!!<br>";
         echo "First delete all users, then remove the group.<br>";
      }
      else { 
         // just in case, test directory
         if ( !is_dir("$archive_dir/$groupname/") || !is_writable("$archive_dir/$groupname/") )
            echo "Error: main archive dir not accessable.";
         else {         
            // delete group directory from filesystem
            exec(escapeshellcmd("rm -Rf $archive_dir/$groupname/"));
            echo "<p>Deleted group dir in filesystem!";
         }
         // remove given group entry from database
         $db_remove_query = "DELETE FROM groups WHERE groupid=$groupid";
         db_exec($db_remove_query);
         echo "<p>Group <b>$groupname</b> is now deleted!";
      }
   }
}

////
// displays form to enter new groupname
function new_group_form ($groupname) {
?>
<form method="post" action="<?php echo $GLOBALS["PHP_SELF"] ?>">
<table>
<tr>
<td>Group Name :</td><td><input type="text" name="groupname" tabindex=10 value="<?php $groupname?>" maxlength=20></td>
</tr>
<tr>
<td></td><td><input type="submit" name="submit" tabindex=20 value="Add Group!"></td>
</tr>
</table>
</form>
<?php  
}

////
// displays form which allows groupname to be changed
function modify_group_form ($groupid, $groupname) {
   global $PHP_SELF;

   echo "<form method='post' action='$PHP_SELF'>\n";
   echo "<input type='hidden' name='groupid' value='$groupid'>\n";
 
   echo "<table>\n";
   echo "<tr><td>Group Name:</td>\n";
   echo "<td><input type='text' name='groupname' value='$groupname'></td></tr>\n";
   echo "</table>\n";
   echo "<input type='submit' name='submit' value='Modify Group'></form>\n";
 
}


printheader();
// when the 'Add a new Group' button has been chosen: 
if ($add == "true")
   new_group_form ("");

// when modify has been pressed:
elseif ($mod == "true")
   modify_group_form ($groupid, $groupname);

else {
   // first handle addition of a new group
   if ($submit == "Add Group!") {
      if ($test = add_new_group ($groupname) ) {
         echo "$test<br>";
         new_group_form ("");
         printfooter ();
         exit;
      }
   }
   // then look whether groupname should be modified
   elseif ($submit =="Modify Group") {
      if ($test = modify_group ($groupid, $groupname)) {
         echo "$test<br>";
         modify_group_form ($groupid, "");
         printfooter ();
         exit;
      }
   } 
  //determine wether or not the remove-command is given and act on it
   elseif ($HTTP_POST_VARS) {
      while((list($key, $val) = each($HTTP_POST_VARS))) {
         if (substr($key, 0, 3) == "del") {
            $delarray = explode("_", $key);
            delete_group($delarray[1]);
         }
      }
   } 


   opendatabase();

   // retrieve all groups and their info from database
   $db_query = "SELECT * FROM groups ORDER BY groupname";
   $db_result = do_database_query($db_query);
   $db_rows = @db_numrows($db_result);

   // print form needed for 'delete' buttons
   echo "<form name='form' method='post' action='$PHP_SELF'>\n";  

   // print header of table
   echo "<table border=\"1\">\n";
   echo "<tr>\n";
   echo "<th>Group</th>";
   echo "<th>Used space</th>\n";
   echo "<th>Admins</th>";
   echo "<th>Users</th>";
   echo "<th colspan=\"2\">Action</th>\n";
   echo "</tr>\n";

   // print all group admins per group in table cells
   for ($i=0; $i<$db_rows; $i++) {
 
      // get results of each row
      $db_row_result = @db_fetch_array($db_result, $i);
      $db_row_groupid = $db_row_result["groupid"];
      $db_row_groupname = $db_row_result["groupname"];
      $db_row_adminid = $db_row_result["adminid"];

      // calculate the amount of space used on disk.  
      if ($space_used = get_group_space ($db_row_groupid))
         $space_used = nice_bytes ($space_used);
      else
         $space_used = "&nbsp;";

      // print start of row of selected group
      echo "<tr>\n";
      echo "<td>$db_row_groupname</td>\n";
 
      // and the amount of diskspace used
      echo "<td>$space_used</td>\n";   

      // get names of users belonging to selected group
      $db_admin_result = get_group_admins($db_row_groupid);
      $db_admin_rows = @db_numrows($db_admin_result);

      // if number of rows greater than zero then print found results
      if ($db_admin_rows > 0) {
         echo "<td>";
         for ($ii=0; $ii<$db_admin_rows; $ii++) {
            $db_admin_row_result = @db_fetch_array($db_admin_result, $ii);
            echo "<b>".$db_admin_row_result["real_name"]."</b><br>";
         }
         echo "</td>\n";
      }
      else
         echo "<td>&nbsp;</td>\n";


      // get names of users belonging to selected group
      $db_user_result = get_group_users($db_row_groupid);
      $db_user_rows = @db_numrows($db_user_result);

      // if number of rows greater than zero then print found results
      if ($db_user_rows > 0) {
         echo "<td>";
         for ($ii=0; $ii<$db_user_rows; $ii++) {
            $db_user_row_result = @db_fetch_array($db_user_result, $ii);
            echo "<b>".$db_user_row_result["real_name"]."</b><br>";
         }
         echo "</td>\n";
      }
      else
         echo "<td>&nbsp;</td>\n";


      // print last columns with links to adjust group
      echo "<td><a href='$PHP_SELF?mod=true&groupid=$db_row_groupid&groupname=$db_row_groupname'>Modify</a></td>\n";
      $delstring = "<input type=\"submit\" name=\"del_" . $db_row_groupid . "\" value=\"Remove\" ";
      $delstring .= "Onclick=\"if(confirm('Are you sure the group $db_row_groupname ";
      $delstring .= "should be removed?')){return true;}return false;\">";                                           
      echo "<td align='center'>$delstring</td>\n";
      //echo "<td><a href='$PHP_SELF?del=true&groupid=$db_row_groupid'>Delete</a></td>\n";
      echo "</tr>\n";
   }

   // print footer of table
   echo "</table>\n";
   echo "<p><a href=\"$PHP_SELF?add=true\">Add a Group</a></p>";
   echo "</form>\n";

   closedatabase();
}

printfooter();

?>
