<?php

// channel_inc.php - Reads and writes channel data
// channel_inc.php - author: SVI Group University Utrecht and Nico Stuurman
  /***************************************************************************
  * Created 4-9-2000 by Nico                                                 *
  * Functions to implement multiple channels for 3-D images                  *
  *                                                                          *
  ****************************************************************************
  *               FUNCTION LIST                                              *
  * function create_channels (&$nr_channels)                                 *
  * function get_channels ($id, $type, &$nr_channels) {                      *
  * function get_color ($colorid, $column)                                   *
  * function show_color_box ($selected_id)                                   *
  * function read_channels_form () {                                         *
  * function print_channel_form ($id, $type)                                 *
  * function store_channels ($id, $type)                                     *
  * function print_channels ($id, $type, $screen) {                          *
  * function show_channels ($id, $type, $border)                             *
  *                                                                          *
  * function show_scope ($imageid) {                                         *
  * function print_microscopes ($selected_id)                                *
  * function print_fs_thumb ($selected_rate)                                 *
  * function print_exp_date ($exp_date)                                      * 
  * function print_exp_date_templ ($exp_date)                                * 
  * function buildquery()                                                    *
  ****************************************************************************
  *                                                                          *
  * Authors: SVI group University Utrecht and Nico Stuurman                  *
  * email: stuurman@rulbim.leidenuniv.nl                                     *
  * demo at: https://impi.leidenuniv.nl                                      *
  * Last modified 12/17/00                                                   *
  * Copyright (c) 2000 by Nico Stuurman                                      *
  * ------------------------------------------------------------------------ *
  *  This program is free software; you can redistribute it and/or modify it *
  *  under the terms of the GNU General Public License as published by the   *
  *  Free Software Foundation; either version 2 of the License, or (at your  *
  *  option) any later version.                                              *
  \**************************************************************************/ 


$channel_fields = "channelid,number,ex,em,exphtcnt,color,pinholeradius,";
$channel_fields .= "type,ext_id";

////
// !fills first channel to create a channel.
function create_channels (&$nr_channels) {
   $channels[0]["ex"]=0;
   $channels[0]["em"]=0;
   $channels[0]["exphtcnt"] = 0 ;
   $channels[0]["pinholeradius"] = 0;
   $channels[0]["color"]=0;
   $nr_channels=1;
   return $channels;
}

////
// !fill array channels with database data, data are stored as a string 
// in channel_data-adjusted
function get_channels ($id, $type, &$nr_channels) {
   global $channel_fields;
   
   // for images
   if ($type ==1) {
      $db_query = "SELECT channels, channel_data FROM images WHERE imageid=$id;";
   }
   // for templates
   elseif ($type == 2) {
      $db_query = "SELECT channels, channel_data FROM template WHERE templateid=$id;";
   }
   if ($db_result=do_database_query($db_query)) {
      if (db_NumRows($db_result)==1) {
         $db_array = db_fetch_array($db_result, 0);
         $channel_data = $db_array["channel_data"];
         if ($channel_data) {
            $channels = unserialize ($channel_data);
            $nr_channels = $db_array["channels"];
         }
         return $channels;
      }
   }
   return false;
} 
      


////
// !get color info
function get_color ($colorid, $column) {
   if ($colorid) {
      $query = "SELECT $column FROM colors WHERE colorid=$colorid;";
      if ($result = db_exec($query) )
         if (db_NumRows ($result)==1)
            return trim (db_Result ($result,0,$column));
   }
   // when something failed
   return false;
}

////
// !displays box for colors
function show_color_box ($selected_id,$chnr) {

   $query = "SELECT colorid,color_string,color_hex,color_text,color_sort FROM colors ORDER BY color_sort;";  
   $result= db_exec ($query);
   if ($result) {
      echo "<SELECT name='colr_$chnr'>\n";
      echo "<OPTION label=none value=0 style='text-allign:center;'>none</OPTION>\n";
      for ($i=0; $i<db_NumRows($result); $i++) {
         $text = db_Result($result,$i,"color_text");
         $id = db_Result($result,$i,"colorid");
         $color = db_Result ($result, $i, "color_hex");
         if ($selected_id == $id)
            $string = "<OPTION selected ";
         else
            $string = "<OPTION ";
         $string .= "label='$text' value='$id' ";
         $string .= "style='text-align:center; background-color:$color; color:$color;'>$text</OPTION>\n"; 
         echo $string;
      }
      echo "</SELECT>\n";
   }
}



////
// !reads variables from form and returns them as a 'channels-array'
function read_channels_form () {
   global $HTTP_POST_VARS;
   
   if ($HTTP_POST_VARS) {
      reset ($HTTP_POST_VARS);
      // read in variables from the form 
      while (list($key, $val) = each ($HTTP_POST_VARS)) {
         switch ($index = substr($key, 0, 4)) {
            case "exwa":
               $index = explode ("_", $key);
               $channels[$index[1]]["ex"] = (int) $val;
               break;
            case "emwa": 
               $index = explode ("_", $key);
               $channels[$index[1]]["em"] = (int) $val;
               break;
            case "expt": 
               $index = explode ("_", $key);
               $channels[$index[1]]["exphtcnt"] = (int) $val;
               break;
            case "pinh": 
               $index = explode ("_", $key);
               $channels[$index[1]]["pinholeradius"] = (float) $val;
               break;
            case "colr": 
               $index = explode ("_", $key);
               $channels[$index[1]]["color"] = (int) $val;
               break;
         }
      }
   return $channels;
   }
}

////
// !displays form with channel data.  needs to be called from within form
function print_channel_form ($id, $type) {
   global $nr_channels, $resubmit, $templateid;

   // if the form is invoked for the first time, try to get the data from the database
   if ($resubmit) {
      // get channels from $HTTP_POST_VARS
      $channels = read_channels_form (); 
   }
   else {
      if ($id) {
         if ($channels = get_channels ($id, $type, $nr_channels) )
            $read_channels = sizeof ($channels);
      } 
      // when image, try to get channel info from template
      elseif ( ($type==1)  && $templateid ) {
            $channels = get_channels ($templateid, 2, $nr_channels);
         
      }
      else {   // no id, so probably a new template/image
         $channels = create_channels($nr_channels);
      } 
   }

   if (!$channels)
      $channels = create_channels($nr_channels);

   // make extra channels when nr_channels has been increased
   $old_size = sizeof ($channels);
   if ( $old_size < $nr_channels) {
      for ($i = $old_size; $i < $nr_channels; $i++) {
         $channels[$i]["ex"]=0;
         $channels[$i]["em"]=0;
         $channels[$i]["exphtcnt"] = 0 ;
         $channels[$i]["pinholeradius"] = 0;
         $channels[$i]["color"]=0;
      }
   }       
   // delete channels them when nr_channels has been reduced
   while (sizeof($channels)>$nr_channels){
      array_pop($channels);
   }

   // and display in table

   echo "<HR>\n";
   echo "<H5><A name='channels'>Channel specific information</A></H5>\n";
   echo "Set the number of channels here:&nbsp&nbsp\n";
   echo "<SELECT NAME='nr_channels'>";
   for ($i=1; $i<13; $i++) {
      if ($i == $nr_channels)
         echo "<OPTION selected>$i</OPTION>\n";
      else
         echo "<OPTION>$i</OPTION>\n"; 
   }
   echo "</SELECT>\n";
   echo "&nbsp&nbsp<INPUT TYPE='SUBMIT' NAME='channel_up' VALUE='Change'>\n"; 
   echo  "<TABLE class='channel' border=1>\n";
   echo  "<TR><TH class='channel'>nr.</TH><TH>Ex.(nm)</TH><TH>Em.(nm)</TH><TH>Photons</TH><TH>Pinhole</TH><TH>Color</TH></TR>\n";
   for ($i=0; $i< (sizeof($channels)); $i++) {
      $number = $channels[$i]->number;
      $ex= $channels[$i]["ex"];
      $em= $channels[$i]["em"];
      $exphtcnt = $channels[$i]["exphtcnt"];
      $pinholeradius = $channels[$i]["pinholeradius"];
      $the_channel = $i+1;
      echo "<TR><TH>$the_channel</TH>\n";
      echo "<TD><INPUT TYPE='TEXT' NAME='exwa_$i' style='text-align:right' VALUE='$ex' SIZE=5 MAXLENGTH=4></TD>\n";
      echo "<TD><INPUT TYPE='TEXT' NAME='emwa_$i' style='text-align:right' VALUE='$em' SIZE=5 MAXLENGTH=4></TD>\n";
      echo "<TD><INPUT TYPE='TEXT' NAME='expt_$i' style='text-align:right' VALUE='$exphtcnt' SIZE=5 MAXLENGTH=1></TD>\n";
      echo "<TD><INPUT TYPE='TEXT' NAME='pinh_$i' style='text-align:right' VALUE='$pinholeradius' SIZE=8 MAXLENGTH=8></TD>\n";
      echo "<TD>";
      show_color_box ($channels[$i]["color"], $i);
      echo "</TD></TR>\n";
   }
   echo "</TABLE>\n";
   echo "<HR>\n";
}
      

////
// !stores all channels in database.  Should be called within transaction
function store_channels ($id, $type) {

   // get data from the form
   $channels = read_channels_form();
   // ignore if teh user changed the nr_channels box while submitting
   $nr_channels = sizeof ($channels);
   if ($nr_channels) {
      $string = serialize ($channels);
      // for images:
      if ($type==1) 
         $db_query = "UPDATE images SET channels='$nr_channels', channel_data='$string' WHERE imageid=$id;";
      // for templates
      elseif ($type==2) 
         $db_query = "UPDATE template SET channels='$nr_channels', channel_data='$string' WHERE templateid=$id;";
      // wrong call:
      else 
         return false;
      // now update the database
      if (do_database_query($db_query))
         return true;
   }
   // if we made it here, it worked
   return true;  
}

////
// !displays channel information in table
function print_channels ($id, $type, $screen) {
   if ($channels = get_channels ($id, $type, $dummy) ) {
      if ($screen)
         $echo .= "<TABLE class='channel' border=1>\n";
      else
         $echo .= "<TABLE class='channel' border=0 align=center>\n";
      $echo .= "<CAPTION><EM>Channel-specific metadata</EM></CAPTION>\n";
      $echo .= "<TR><TH class='channel'>nr.</TH><TH>Ex.</TH><TH>Em.</TH><TH>Photons</TH><TH>Pinhole</TH><TH>Color</TH></TR>\n";
      for ($i=0; $i<sizeof($channels); $i++) {
         $number = $i + 1;
         $ex= $channels[$i]["ex"];
         $em= $channels[$i]["em"];
         $exphtcnt = $channels[$i]["exphtcnt"];
         $color = get_color ($channels[$i]["color"],"color_hex");
         $color_text= "style='background-color:$color; color:$color;'";
         $pinholeradius = $channels[$i]["pinholeradius"];
         $echo .= "<TR><TH>$number</TH><TD>$ex</TD><TD>$em</TD><TD>$exphtcnt</TD>";
         $echo .= "<TD>$pinholeradius</TD><TD $color_text>$color</TD></TR>\n";
      }
      $echo .= "</TABLE>\n";
      return $echo;
   }
   return false;
}

////
function show_channels ($id, $type, $screen) {
   $string = print_channels ($id, $type, $screen);
   echo $string;
}

////
// !Returns a string with microscope type of an image
function show_scope ($imageid) {
   if ($imageid) {
      $query = "SELECT microscope FROM microscopes WHERE microscopeid IN (SELECT ";
      $query.= "microscope_type FROM images WHERE imageid='$imageid');";
      if ($result = db_exec ($query)) {      
         $numrows = db_NumRows ($result);
         if ($numrows==0 || $numrows > 1)
            return false;
         else return db_result($result, 0, "microscope");
      }
      else return false;
   }
   else return false;
}


////
// !Displays selection box for microscopes. 
// Needs to be called from within a form and table
function print_microscopes ($selected_id, $norow, $color = "") {
   $query = "SELECT microscopeid,microscope FROM microscopes;";
   $result = do_database_query ($query);
   $numrows = db_NumRows ($result);

   if ($numrows==0)
      return false;
   
   if (!$selected_id) $selected_id = 0;
   $message = translate ("sensortype"); 
   if ($norow)
      echo  "<TD ROWSPAN=3 valign='top' $color>$message<br>\n";
   else
      echo "<TR><TD $color>$message</TD>\n";
   if ($norow)
      echo "<SELECT size=\"$numrows\" name=\"scope[]\">\n";
   else
      echo "<TD><SELECT size=1 name=\"scope[]\">\n";

   for ($i=0; $i < $numrows; $i++) {
      $microscopeid = db_result($result, $i, 'microscopeid');
      $microscope = db_result ($result, $i, 'microscope');
      if ($microscopeid==$selected_id) 
         echo "<OPTION selected value=\"$microscopeid\">$microscope</OPTION>\n";
      else 
         echo "<OPTION value=\"$microscopeid\">$microscope</OPTION>\n";
   }
   
   echo "</SELECT></TD>\n";
   return true;
}

////
// !Displays selection box for thumbnail movie's frame rate. 
// Needs to be called from within a form and table
function print_fs_thumb ($selected_rate) {
   if (!$selected_rate)
      $selected_rate = 13;
   
   echo "<TD>&nbsp &nbsp (z) frames/sec </TD>\n";
   echo "<TD><SELECT size=1 name=\"fs[]\">\n";

   for ($i=1; $i < 13; $i++) {
      if ($i <> 7 && $i <> 9 && $i <> 11) {
         $rate = 30/$i;
         if ($i == $selected_rate)
            echo "<OPTION selected value=\"$i\">$rate</OPTION>\n";
         else
            echo "<OPTION value=\"$i\">$rate</OPTION>\n";
      }
   }

   if ($selected_rate==13) 
      echo "<OPTION selected value=\"13\">No Movie</OPTION>\n";
   else
      echo "<OPTION value=\"13\">No Movie</OPTION>\n";
 

   echo "</SELECT></TD>\n";
   return true;
}
 
////
// !Displays expiration date entry field. 
// Needs to be called from within a form and table
function print_exp_date ($exp_date) {
   global $date_format;

   if ($exp_date) 
      $date_string = date ($date_format, $exp_date);
   else {
      if ($date_format == "d-m-Y")
         $date_string = "dd-mm-yyyy";
      elseif ($date_format == "m-d-Y")
         $date_string = "mm-dd-yyyy";
      $checked = "checked";
   }
   echo "&nbsp&nbsp&nbsp&nbsp&nbsp <b>Delete</b> image after: ";
   echo "<input type='text' name='exp_date_text' size='11' maxlength='11'
         value=\"$date_string\">\n";
   echo "&nbsp or <input type='checkbox' name='exp_date_box' $checked>"; 
   echo "&nbsp Never delete this image</td>\n";
}

////
// !Displays expiration date entry field for templates
// Needs to be called from within a form and table
function print_exp_date_templ ($exp_period, $string1="", $never_box=true) {
   global $date_format;
   
   if ($exp_period < 604800) {
      $day = "selected";
      $numberof = $exp_period/86400;
   }
   elseif ($exp_period < 2419200) {
      $week = "selected";
      $numberof = $exp_period/604800;
   }
   elseif ($exp_period < 31536000) {
      $month = "selected";
      $numberof = $exp_period/2419200;
   }
   else {
      $year = "selected";
      $numberof = $exp_period/31536000;
   } 

   if ($string1)
      echo "$string1";
   else
      echo "&nbsp &nbsp &nbsp &nbsp <b>Delete</b> image after: ";
   echo "<input type='text' name='exp_number' size=2 maxlength=2
          value=$numberof>\n";
   echo "<select name='exp_unit' size=1>\n";
   echo "<option $day value=86400>day(s)</option>\n";
   echo "<option $week value=604800>week(s)</option>\n";
   echo "<option $month value=2419200>month(s)</option>\n";
   echo "<option $year value=31536000>year(s)</option>\n";
   echo "</select>\n";
 
   if ($never_box) {
      if ($exp_period < 86300) 
         $checked = "checked"; 
      echo "&nbsp or <input type='checkbox' name='exp_never_box' $checked>"; 
      echo "&nbsp Never delete the image</td>\n";
   }
}

////
//This function helps to construct SQL queries with numbers.
function logicalnumber ($field, $number, $prefix="",$sufix="") {


    if ($number!="" && $field) {

         // Correct bad typing 5. -> 5.0
         $number = preg_replace("/\.$/", ".0", $number);
         $number = preg_replace("/\.-/", ".0-", $number);

         switch (true):

         case is_numeric($number):
         // fixed numerical value
            $query .= " $field = $number ";
            break;
         
         case preg_match("/^([0-9]*\.*[0-9]+)(-)([0-9]*\.*[0-9]+)$/", $number):
         // limited range: 10.1-30.5
            $tmpline = preg_replace("/([0-9]*\.*[0-9]+)(-)([0-9]*\.*[0-9]+)/",
                      "between \\1 and \\3", $number);
            $query .= " $field $tmpline ";
            break;
         case preg_match("/^([0-9]*\.*[0-9]+)(-)$/", $number):
         // low limited range: 10.5-
            $tmpline = preg_replace("/([0-9]*\.*[0-9]+)(-)/",
                      ">= \\1", $number);
            $query .= " $field $tmpline ";
            break;
         case preg_match("/^(>|<|>=|<=|=|!=|<>)([0-9]*\.*[0-9]+)$/", $number):
         // comparison operators: > >= = != <>
            $query .= " $field $number ";
            break;
         default:
         // POSIX regular expression
            $number = str_replace(array(".","*","?","+"),
                                 array("\.",".*",".","\+"),$number);
            if (strpos($number, '!') === 0) {
               $oper="!~"; 
               $elem=substr($number,1);
            } else {
               $oper="~";
               $elem=$number;
            }

            $query .= " $field $oper '^$elem$' ";         
         endswitch;
         $query = " ".$prefix . $query . $sufix." ";
         return $query;
    } else return "";
}

////  
//This function helps to construct SQL queries for channels.
function logicalchan ($field, $number, $chan, $prefix="",$sufix="") {
         
            
    if ($number!="" && $field && $number!="ANY") {

         // Correct bad typing 5. -> 5.0
         $number = preg_replace("/\.$/", ".0", $number);
         // POSIX regular expression
         $number = str_replace(array(".","*","?","+"),
                                 array("\.",".*",".","\+"),$number);

         if (strpos($number, '!') === 0) {
            $oper="!~";
            $elem=substr($number,1);
         } else {
            $oper="~";
            $elem=$number;
         }
         
         if (is_numeric($chan)) {
         // If we look for data in a specific channel...
            $chan=$chan-1;
            // How to make queries on a specific channel?
            // We look for the occurrence of the string of interest with no
            // other "array" identifier (a:) after the channel number.
            // So i:2;a:5:{s:2 ... "ex";i:364 will match, but
            //    i:2;a:5:{s:2  ... a: ... "ex";i:364  will not
            // thus linking the channel number and the searched field.
            $query = " channel_data $oper 'i:$chan;a:([^a]|a[^:])*";
            $query .= "\"$field\";.:$elem' ";


         } else {
            $query = " channel_data $oper '\"$field\";.:$elem;' ";
         }
               
         $query = " ".$prefix . $query . $sufix." ";                  
         return $query;                                       
    } else return "";                                         
}  

////
// This function builds a SQL query, to search the database.
function buildquery() {

   global $S_USERID, $S_GROUPID, $S_STATUS, $G_SUPER, $S_LOGIN, $file_name, $trust_array;
   global $trust, $loginname, $date_day, $date_month, $project_array;
   global $date_year, $date_comparison, $des_cription, $keys, $keybool;
   global $action, $audit_array, $audit, $period, $period_nr, $file_id;
   global $namebool, $sortby, $searchtype, $file_x, $file_y, $file_z;
   global $file_thumbx, $file_thumby, $file_thumbt, $file_samplex;
   global $file_sampley, $file_samplez, $file_samplet, $file_NA, $file_lri;
   global $file_mri, $file_numchan, $file_inchan, $file_ex, $file_em;
   global $file_photon, $file_ph, $microscope, $file_color, $file_raw;


   $audit_groups = get_audit_groups($S_USERID);
   // determine id of user if any was chosen
   if (!$trust)
      $trust = get_array_value ($trust_array);

   // determine id of 'trusted user' if any was chosen
   if (!$audit)
      $audit = get_array_value ($audit_array);
   
   // if any of these columnnames changes please beware of result array (names are used as headers)
   $query = "SELECT I.imageid, I.filename,  I.filesize, I.filetype, U.real_name, U.email, I.date_time, I.thumb_nr, I.compressed, U.login FROM images AS I, users AS U ";
   $query .= "WHERE I.userid= U.userid AND ( ( (I.private = '0') AND (I.restrict_to_group = '0' OR ";
   $query .= "($S_GROUPID = (SELECT groupid FROM USERS WHERE userid = I.userid) )";
   // add groups that can be audited
   if (is_array($audit_groups)){
      for ($i=0; $i<sizeof($audit_groups);$i++)
         $query .= " OR ($audit_groups[$i] = (SELECT groupid FROM users WHERE userid = I.userid)) ";
   }
   $query .= ")) OR I.userid='$S_USERID' ";
   // trusted users are OK
   $query .= " OR I.trust='$S_USERID' ";
   // SuperUser may see it all...
   if ($S_STATUS == $G_SUPER)
      $query .= "OR TRUE";
   $query .= ")";

   if ($file_name != ""):
      // if the filename field has been entered this is added to the query
      // POSIX regular expression
       $file_name_string = str_replace(array(".","*","_","?","+"),
               array("\\\.",".*","\_",".","\\\+"),$file_name);
       // This \\\. is for postgres, PHP just uses \.

      $file_name_string = strtoupper ($file_name_string);
      $tok = strtok ($file_name_string," ");                      

    $query = $query . " AND ( ";
      
      while ($tok) { 
         if (strpos($tok, '!') === 0) {
              $oper="!~";
              $elem=substr($tok,1);
         } else {
              $oper="~";
              $elem=$tok;
         }                                         
         $query = $query . 
              "UPPER (I.filename) $oper '$elem' $namebool ";
         $tok = strtok (" ");                                 
      }  
      // this is just to complete the sentence:
      if ($namebool=="or") $query = $query . "false)";
      else $query = $query . "true)";
   endif;


    if ($file_id != ""):                                     
                                                               
       $tok = strtok ($file_id," ");                      
       $query = $query . " AND (";
       while ($tok) {                                          
         $query .= logicalnumber("I.imageid",$tok,"","OR");
         $tok = strtok (" ");
      }
       $query = $query . " false)";  // this is just to complete the sentence
    endif;  


     
   if ($action == "mypics") {
      $loginname=$S_LOGIN;
      if ($loginname !="")
         $query.= " AND (U.login = '$loginname')";
   }

   // deal with login names
   if ($trust)
      $query .= " AND (U.userid = '$trust')";  

   // deal with users specifically trusted to see an image
   if ($audit)
      $query .= " AND (I.trust = '$audit')";  

   // date-related thingies
   if ($period) {
      if ($period && !$period_nr) 
         $period_nr = 1;   
      if ($period == "day") $period_sec  =     86400 * $period_nr;
      if ($period == "week") $period_sec  =   604800 * $period_nr;
      if ($period == "month") $period_sec =  2592000 * $period_nr;
      if ($period == "year") $period_sec =  30758400 * $period_nr;
   }

   // if day/month/year variables are set then these are added to the query
   if (($date_day != "") && ($date_month != "") && ($date_year != "")) {      
      $date_start = mktime( 0, 0, 0, $date_month, $date_day, $date_year);
      $date_end = mktime( 24, 60, 60, $date_month, $date_day, $date_year);
   
      if ($date_comparison == "on") {
         $date_start -= $period_sec;
         $date_end += $period_sec;
         $query .= " AND I.date_time >= '" . $date_start . "' AND I.date_time <=  '" . $date_end . "'";
      }
   
      if ($date_comparison == "after") {
         if ($period_sec) {
            $date_end += $period_sec;
            $query .= " AND I.date_time >= '" . $date_start . "' AND I.date_time <=  '" . $date_end . "'"; 
         }
         else
            $query .= " AND I.date_time >= '" . $date_start . "'";
      }
      
      if ($date_comparison == "before") {
         if ($period_sec) {
            $date_start -= $period_sec;
            $query .= " AND I.date_time >= '" . $date_start . "' AND I.date_time <=  '" . $date_end . "'";
         }
         else
            $query .=  " AND I.date_time <= '" .  $date_end . "'";
      }
   }

   if ($des_cription != "") {

      $file_desc_string = str_replace(array(".","*","_","?","+"),
               array("\.",".*","\_",".","\+"),$des_cription);

      $file_desc_string = strtoupper ($file_desc_string);
      $tok = strtok ($file_desc_string," ");

      while ($tok) {
         if (strpos($tok, '!') === 0) {
            $oper="!~";  
            $elem=substr($tok,1);
         } else {
            $oper="~";
            $elem=$tok;
         }
         $query = $query .
              " AND UPPER (description) $oper '$elem' ";
         $tok = strtok (" ");
      }
   } 

   // Add keyword information to query
   if ($keys && $keys[0]) {
      if ($keybool=="or") {
         reset ($keys); 
         while (list(, $value) = each ($keys)) {
            $string .= $value.", ";
         }
         $string = substr($string, 0, strlen($string) - 2);
         $query .= "AND (imageid IN (SELECT imageid FROM imagexkeywords ";
         $query .= "WHERE keywordid IN ($string) ) ) ";
      }
      if ($keybool=="and") {
         reset ($keys); 
         while (list(, $value) = each ($keys)) {
	    $query .= "AND (imageid IN 
                       (SELECT imageid FROM imagexkeywords WHERE ";
            $query .= "keywordid = $value) ) ";
         }
      }   
   }

   // add projects. They are passed as an array and have to be 'translated'
   // into imageids.  For simplicity, the imageids are pulled from the 
   // database directly (instead of nesting into the SQL statement 
   if (is_array($project_array) && $project_array[0]) {
      $subquery = "SELECT imageid FROM projectximages WHERE "; 
      for ($i=0; $i<sizeof($project_array); $i++) {
         $pid = $project_array[$i];
         $subquery .= "projectid=$pid ";
         if ($i < sizeof ($project_array) - 1)
            $subquery.= "OR ";
      }
      $result = do_database_query ($subquery);
      if ($result)
         $size_result = db_NumRows ($result);
      if ($size_result) {    
         $query .= "AND (I.imageid IN (";
         for ($i=0; $i<$size_result; $i++) {
            $result_row = db_Fetch_Array ($result, $i);
            $iid = $result_row ["imageid"];
            if ($i<$size_result-1)
               $query .= "$iid,";
            else
                $query .= "$iid";
         }
         $query .= ") ) ";
      }
      else
         $query .= "AND I.imageid=0 ";
   }


   if ($searchtype!="ANY" && $searchtype!="") {
        $query .= " AND filetype='$searchtype' ";
   }

   $microscope_type = get_array_value ($microscope);
   if ($microscope_type!="ANY" && $microscope_type!="") {
        $query .= " AND microscope_type='$microscope_type' ";
   }  

   if ($file_raw!="ANY" && $file_raw!="") {
        $query .= " AND raw_data='$file_raw' ";
   }


   $query .= logicalnumber("x_thumb", $file_thumbx, "AND", "");
   $query .= logicalnumber("y_thumb", $file_thumby, "AND", "");
   $query .= logicalnumber("fs_thumb", $file_thumbt, "AND", "");
   $query .= logicalnumber("x_nm", $file_samplex, "AND", "");
   $query .= logicalnumber("y_nm", $file_sampley, "AND", "");
   $query .= logicalnumber("z_nm", $file_samplez, "AND", "");
   $query .= logicalnumber("x_pixel", $file_x, "AND", "");
   $query .= logicalnumber("y_pixel", $file_y, "AND", "");
   $query .= logicalnumber("z_pixel", $file_z, "AND", "");
   $query .= logicalnumber("t_sec", $file_samplet, "AND", "");
   $query .= logicalnumber("numaperture", $file_NA, "AND", "");
   $query .= logicalnumber("refrinxlensmedium", $file_lri, "AND", "");
   $query .= logicalnumber("refrinxmedium", $file_mri, "AND", "");
   $query .= logicalnumber("channels", $file_numchan, "AND", "");
   $query .= logicalchan("ex", $file_ex, $file_inchan, "AND", "");
   $query .= logicalchan("em", $file_em, $file_inchan, "AND", "");
   $query .= logicalchan("exphtcnt", $file_photon, $file_inchan, "AND", "");
   $query .= logicalchan("pinholeradius", $file_ph, $file_inchan, "AND", "");
   $query .= logicalchan("color", $file_color, $file_inchan, "AND", "");



  if ($sortby=="id") $query = $query." ORDER BY imageid DESC;";
   elseif ($sortby=="name") $query = $query." ORDER BY filename ASC;";
   else $query = $query." ORDER BY date_time DESC;";
   // echo "$query<Br>";
   Debug::printDebug("Query: $query");
   return $query;
}


?>
