<?php

  // class-auth.php - Takes care of authorization.  
  // class-auth.php - author: Nico Stuurman  

  /***************************************************************************
  * This file handles the authorization popup box for logging in to the      *
  * database as a user, groupadmin or sysadmin. The file is called from      *
  * within config_inc.php so every page is only visible after logging in.    *
  * This version doesnot use http popup boxes, but session and cookies       *
  *                                                                          *
  * function Auth()
  * function allowonly($utype)                                               *
  * function loginscreen ($message)                                          *
  *                                                                          *
  * Authors: SVI group University Utrecht and Nico Stuurman                  *
  * email: nicos@itsa.ucsf.edu                                               *
  * demo at: https://impi.leidenuniv.nl                                      *
  * Copyright (c) 2000, 2001 by Nico Stuurman                                *
  * ------------------------------------------------------------------------ *
  *  This program is free software; you can redistribute it and/or modify it *
  *  under the terms of the GNU General Public License as published by the   *
  *  Free Software Foundation; either version 2 of the License, or (at your  *
  *  option) any later version.                                              *
  \**************************************************************************/

/* Modified by Patrick Colm Audley (PCA) <paudley@blackcat.ca> to include NIS
   logins in preference to database logins and to include auto creation of users
   and groups based on NIS information.  Crypted password storage also added */

/* Modified by Frank Keijzers (SVI) <frank@svi.nl> to have authentication work 
   from a class. This would make adjustments much easier to make and to 
   overview how the authentication actually works. */

/* Some corrections by Jose Vina (SVI) <jose@svi.nl> */

#require("$include_dir/class-client.php");

class Auth
{
    function Auth()
    {
	// Global vars we need to have this thing work like it was before we 
	// started the use a class for authentication.

        global $PHP_AUTH_USER;
        global $PHP_AUTH_PW;

        global $use_sessions;
        global $auth_method;
        global $nis_domain;

        global $username;
        global $password;

        global $impose_folder;
        global $infolder;
        global $people_dir;
        global $outfolder;


        global $S_BROWSER;
        global $S_USERID;
        global $S_NAME;
        global $S_PAMAUTH;
        global $S_LOGIN;
        global $S_GROUPID;
        global $S_STATUS;
        global $S_EMAIL;
        global $S_HOMEDIR;
        global $S_OUTDIR;
        global $S_SETTINGS;


	// start new client class for browser info
	$client = new cl_client;

	// protect from outside variables
	$auth=false;

	//$authfaile = false;
	if ($use_sessions)
	{
	    if (function_exists("session_cache_limiter"))
	    {
		session_cache_limiter("private");
	    }
	    session_start();


            // if this is a login, authenticate the user:
	    if ($_POST["logon"])
	    {
		$PHP_AUTH_USER=$_POST["user"];
		$PHP_AUTH_PW=$_POST["pwd"];
		if ($PHP_AUTH_USER && $PHP_AUTH_PW)
		{

		    // PCA:
		    //
		    // Do NIS checks first. If they are a NIS user, ignore any 
		    // password in the database in favour of their NIS password
		    // to facilitate single sign on.
		    if ($auth_method=="NIS" && $nis_domain != "")
		    {
			$pwent = @yp_match($nis_domain, "passwd.byname", $PHP_AUTH_USER);
			if ($pwent != false)
			{
			    // PCA: We have them in the NIS map
			    list($username,$password,$uid,$gid,$gecos,$home,$shell) = preg_split('/:/',$pwent);
			    if (check_password($PHP_AUTH_PW,$password))
			    {
				$auth = true;
				// PCA: Extract the groupname from NIS.
				$grpent = @yp_match($nis_domain,"group.bygid",$gid);
				list($groupname,$rest) = preg_split('/:/',$grpent);

				// PCA: Make a group if they don't have one.
				$db_query = "SELECT * FROM groups where groupname = '$groupname';";
				$db_result = db_exec($db_query);
				$db_rows = db_numrows($db_result);
				if ($db_rows != 1 && strtolower($user_autocreate) == "yes")
				{
				    $db_query = "INSERT INTO groups (groupname) VALUES ('$groupname');";
				    $db_result = db_exec($db_query);
				    $db_query = "SELECT * FROM groups where groupname = '$groupname';";
				    $db_result = db_exec($db_query);
				    $db_rows = db_numrows($db_result);
				}
			  
				// PCA: Pull out the groupid.
				$db_row_result = db_fetch_array($db_result, 0);
				$groupid = $db_row_result["groupid"];

				// PCA: If they have a NIS account but no local 
				// account, make one.
				if (strtolower($user_autocreate) == "yes")
				{
				    if (!exist_user($PHP_AUTH_USER))
				    {
					// make a new account
					$db_query = "INSERT INTO users (login, pwd, groupid, real_name, status, active, email, homedir, outdir, login_defunct)  VALUES('$username', NULL, '$groupid', '$gecos', '$G_USER', 't', '', '$home', 'fromsidb', '$username')";
					$db_result = db_exec($db_query);
				    }
				}
			    }
			    else 
			    {
				$authfailed = true;
			    }
			}
		    }
		    // check with PAM
		    elseif ($auth_method=="PAM" && $pam_prg && !$auth)
		    {
			$esc_user = escapeshellarg($PHP_AUTH_USER);
			$esc_pass = escapeshellarg($PHP_AUTH_PW);
			$test = exec ("echo $esc_pass | $pam_prg $esc_user", $dummy,$result);
			if ($result)
			{
			    // we are authenticated
			    if (!exist_user($PHP_AUTH_USER))
			    {
				if (strtolower($user_autocreate)=="yes")
				{
				    // make a new account
				    $db_query = "INSERT INTO users (login, pwd, groupid, real_name, status, active, email, homedir, outdir, login_defunct)  VALUES('$PHP_AUTH_USER', '', '$auto_group', '$PHP_AUTH_USER', '$G_USER', 't', '', '$home', 'fromsidb', '$PHP_AUTH_USER')";
				    $db_result = db_exec($db_query);
				} 
			    }
			    if (exist_user($PHP_AUTH_USER))
			    {
				$auth = true; 
				$S_PAMAUTH = true;
				$_SESSION["S_PAMAUTH"]=$S_PAMAUTH;
			    }
			}
		    }

		    // check submitted login and passwd in SQL database
		    if (!$auth)
		    {
			$pwd_md5=md5($PHP_AUTH_PW);
			$db_query = "SELECT login FROM users WHERE login='$PHP_AUTH_USER' AND pwd='$pwd_md5'";
			$db_result = db_exec($db_query);
			$db_rows = db_numrows($db_result);
			if ($db_rows == 1)
			{
			    $auth=true;
			}
		    }
	     
		    // if authenticated, this session is OK:
		    if ($auth)
		    {
			$_SESSION["PHP_AUTH_USER"]=$PHP_AUTH_USER;
			// the following is not used in sidb, but in phplabware
			if ($_SESSION["javascript_enabled"] || $_POST["javascript_enabled"])
			{
			   $_SESSION["javascript_enabled"]=true;
			}
			else
			{
			    $_SESSION["javascript_enabled"]=false;
			}
			// when the login was secure but user does not wanna stay secure
			if (getenv("HTTPS") && !$_POST["ssl"])
			{
			    // send meta tag redirecting to http page and exit
			    $PHP_SELF=$_SERVER["PHP_SELF"];
			    $server= getenv ("HTTP_HOST");
			    $url="http://$server$PHP_SELF";
			    echo "<html>\n<head>\n";
			    echo "<meta http-equiv='refresh' content=0;URL='$url'>\n";
			    echo "</head>\n</html>";
			    exit();
			} 
		    }
		    else
		    {
			$PHP_AUTH_USER = false;
			Auth::loginscreen("<h4>Your credentials were not accepted, Please try again</h4>");
			exit();
		    }
		}
		else 
		{
		     Auth::loginscreen("<h4>Please enter your username and password</h4>");
		}
	    } 

	    // If the $PHP_AUTH_USER is not set, we need to identify and 
	    // authenticate.
	    if (!$PHP_AUTH_USER)
	    {
		$PHP_AUTH_USER = $_SESSION["PHP_AUTH_USER"];
	    }
	    if (!$PHP_AUTH_USER)
	    {
		// Display logon screen.
		Auth::loginscreen();
	    } 
	    else
	    {
		// we must have been authenticated directly or through the session
		$db_query = "SELECT * FROM users WHERE login='$PHP_AUTH_USER'";
		$db_result = db_exec($db_query);
		if (!$db_result) {
		   echo "Fatal database error 1.<br>";
		   exit();
		}
		$db_rows = db_numrows($db_result);
		// save frequently used variables
		$db_row_result = db_fetch_array($db_result, 0);
		// check whether account allows logins
		$active = $db_row_result["active"];
		if ($active=="t")
		{
		    $S_BROWSER = $client->browser;
		    $S_USERID = $db_row_result["userid"];
		    $S_NAME = $db_row_result["real_name"];
		    $S_PAMAUTH = $_SESSION["S_PAMAUTH"];
		    $S_LOGIN = $db_row_result["login"];
		    $userfolder=strtolower($S_LOGIN);
		    $S_GROUPID = $db_row_result["groupid"];
		    $S_STATUS = $db_row_result["status"];
		    $S_EMAIL = $db_row_result["email"];
		    if ($impose_folder)
		    {
			$dir = $infolder;
			$outdir = $outfolder;
		    }
		    else
		    {
			$dir = $db_row_result["homedir"]; 
			$outdir = $db_row_result["outdir"];
		    }   

		    if ($dir)
		    {
			$S_HOMEDIR = $people_dir . "/".$userfolder."/".$dir;
		    }
		    if ($outdir)
		    {
			$S_OUTDIR = $people_dir . "/" .$userfolder."/".$outdir;
		    }
		    if ($settings = $db_row_result["settings"] ) 
		    {
			$S_SETTINGS = unserialize ($settings);
		    }
		}
		elseif ($active=="f")
		{
		    echo "Inactive user.<br>";
		}
		else
		{
		    echo "Fatal database error 2.<br>";
		}
	    }
	}
	else
	{
	    // use http authentication

	    // set globals, in case register_globals is off:
	    $PHP_AUTH_USER = $_SERVER["PHP_AUTH_USER"];
	    $PHP_AUTH_PW   = $_SERVER["PHP_AUTH_PW"];

	    // check to see if $PHP_AUTH_USER already contains info
	    if (!isset($PHP_AUTH_USER))
	    {
		// if empty, send header causing dialog box to appear
		header('WWW-Authenticate: Basic realm="Scientific Image Database"');
		header('HTTP/1.0 401 Unauthorized');		
		echo "Authorization required.";
		exit();	
	    }
	    else
	    {
		// if not empty, check submitted login and passwd
		// if the pam_prg is present, check whether the user is known on 
		// the system 
		if ($pam_prg)
		{
		    // this only makes sense if the user has an account on sidb
		    $db_query = "SELECT * FROM users WHERE login='$PHP_AUTH_USER'";
		    $db_result = db_exec($db_query);
		    $db_rows = db_numrows($db_result);
		    if ($db_rows != 0)
		    {
			$esc_user = escapeshellarg($PHP_AUTH_USER);
			$esc_pass = escapeshellarg($PHP_AUTH_PW);
			$test = exec ("$pam_prg $esc_user $esc_pass", $dummy,$result);

			if ($result)
			{ 
			    // we are authenticated
			    $auth = true; 
			    $S_PAMAUTH = true;
			}
		    }
		}    
		  
		if ((!($pam_prg && $auth)) && $PHP_AUTH_PW)
		{
		    $pwd_md5=md5($PHP_AUTH_PW);
		    $db_query = "SELECT * FROM users WHERE login='$PHP_AUTH_USER' AND pwd='$pwd_md5'";
		    $db_result = db_exec($db_query);
		    $db_rows = db_numrows($db_result);
		    if ($db_rows != 0)
		    {
			$auth=true;
		    }
		}

		if ($auth)
		{
		    // save frequently used variables
		    $db_row_result = db_fetch_array($db_result, 0);
		    // check whether account allows logins
		    if ($db_row_result["active"]=="t")
		    {
			$S_BROWSER = $client->browser;
			$S_USERID = $db_row_result["userid"];
			$S_NAME = $db_row_result["real_name"];
			$S_LOGIN = $db_row_result["login"];
			$userfolder=strtolower($S_LOGIN);
			$S_GROUPID = $db_row_result["groupid"];
			$S_STATUS = $db_row_result["status"];
			$S_EMAIL = $db_row_result["email"];
			if ($impose_folder)
			{
			    $dir = $infolder;
			    $outdir = $outfolder;
			}
			else
			{
			    $dir = $db_row_result["homedir"]; 
			    $outdir = $db_row_result["outdir"];
			}
			if ($dir)
			{
			    $S_HOMEDIR = $people_dir . "/".$userfolder."/".$dir;
			}
			if ($outdir)
			{
			    $S_OUTDIR = $people_dir . "/" .$userfolder."/".$outdir;
			}
			if ($settings = $db_row_result["settings"] ) 
			{
			    $S_SETTINGS = unserialize ($settings);
			}
		    }
		    else
		    {
			// not an active account, pop up again
			header('WWW-Authenticate: Basic realm="Scientific Image Database"');
			header('HTTP/1.0 401 Unauthorized');		
			echo "Authorization required.";
			exit();
		    }
		}
		else
		{
		    // if failed, pop up again...
		    header('WWW-Authenticate: Basic realm="Scientific Image Database"');
		    header('HTTP/1.0 401 Unauthorized');		
		    echo "Authorization required.";
		    exit();
		}
	    }
	}
    }

    ////
    // !function that checks if user is allowed to view page
    function allowonly ($utype) 
    {
        global $S_STATUS;
        if ($S_STATUS < $utype)
        {
            echo "You do not have the rights to view this page!";
            exit;
        }
    }

    ////
    // !presents the login screen when authenticating witth sessions
    function loginscreen ($message="<h3>Login to the Image Database</h3>")
    {
        global $_SERVER, $safe_login, $secure_server, $imagelink;
        global $id;

        $PHP_SELF=$_SERVER["PHP_SELF"];
        $server= getenv ("HTTP_HOST");

        if (!$server)
        {
            $server=$_SERVER["HTTP_HOST"];
        }
        if ($safe_login && $secure_server)
        {
            $addres="https://$server$PHP_SELF";
        }
        else
        {
            $addres="http://$server$PHP_SELF";
        }

        printheader ();
        Debug::printDebug(" Debug messages are enabled in includes/config.php by ".
                "\$sidb->debug = true;");
        echo "<form name='loginform' method='post' action=$addres onSubmit='this.javascript_enabled.value=true'>\n";
        echo "<input type='hidden' name='logon' value='true'>\n";
        echo "<input type='hidden' name='javascript_enabled' value='0'>\n";
        echo "<input type='hidden' name='id' value='$id'>\n";
        echo "<table align=center>\n";
        echo "<tr><td colspan=2 align='center'>$message</td>\n";
        $imstring = randomimage("frontims");
        if ($imstring);
	{
	    echo "<td rowspan=6>&nbsp;&nbsp&nbsp;$imstring</td>";
	}
        echo "</tr>\n";
        echo "<tr><td>Your login name:</td>\n";
        echo "<td><input name='user' size=10 value=''></td></tr>\n";
        echo "<tr><td>Password:</td>\n";
        echo "<td><input type='password' name='pwd' size=10 value=''></td></tr>\n";
        echo "<tr><td colspan=2 align='center'>";
        if ($secure_server)
	{
	    echo "<input type='hidden' name='ssl' value='on'>\n";
        }
        echo "</td></tr>\n";
        echo "<tr><td colspan=2 align='center'>";
        echo "<input type='submit' name='submit' value='Login'></td></tr>\n";
        echo "</table>\n";
        printfooter();
        exit();
    }
}



/******************** start of global script ********************/

$newauth = new Auth();

?>
