<?php

/*
 * This file contains the Database class.
 *
 */
class Database
{

    var $db_conn;
    var $db_query;
    var $db_result;
    var $db_rows;
    var $db_fields;
    var $db_array;

    function Database($query = NULL)
    {
	$this->db_conn = NULL;
	$this->db_query = NULL;
	$this->db_result = NULL;
	$this->db_rows = 0;
	$this->db_fields = NULL;
	$this->db_array = NULL;

	if (isset($query))
	{
	    $this->setQuery($query);
	}
    }


    function setQuery($query_str)
    {
	$this->db_rows = 0;
	$this->db_fields = 0;
	$this->db_array = NULL;
	$this->db_query = $query_str;
	$this->db_result = NULL;

	if (is_string($query_str) === FALSE)
	{
	    error_log("Invalid string $query_str for setQuery.");
	    exit;
	}
	if ($this->isConnected() === FALSE)
	{
	    $this->setConnection($this->openConnection());
	}
	if ($this->isConnected() === FALSE)
	{
	    error_log("Invalid database connection in setQuery.");
	    return;
	}

	$this->db_result = @pg_query($this->getConnection(),
				   $this->db_query);
	if ($this->db_result === FALSE)
	{
	    error_log("Error while trying to execute database query");
	    error_log("Error in database query: $query_str");
	    exit;
	}
	else
	{
	    $this->db_rows = @pg_num_rows($this->db_result);
	    if ($this->db_rows == -1)
	    {
		error_log("Error while trying to execute database query");
		error_log("Numrows fails after <b>$query_str</b>.");
		exit;
	    }
	    $this->db_fields = @pg_num_fields($this->db_result);
	    if ($this->db_fields == -1)
	    {
		error_log("Error while trying to execute database query");
		error_log("Numfields fails after <b>$query_str</b>.");
		exit;
	    }
	}
    }

    function getQuery()
    {
	return $this->db_query;
    }

    function queryFields()
    {
	return $this->db_fields;
    }

    function queryRows()
    {
	Debug::printDebug("queryRows: ".$this->db_rows);
	return $this->db_rows;
    }

    function queryRowsAffected()
    {
	return @pg_affected_rows($this->db_result);
    }

    function queryResult()
    {
	return $this->db_result;
    }

    function queryArray($row)
    {
	$this->db_array = @pg_fetch_array($this->db_result, $row, PGSQL_BOTH);
	if ($this->db_array === FALSE)
	{
	    error_log("Error in attempting to execute database query");
	    error_log("Can't fetch result array after <b>$this->db_query</b>.");
	}
	return $this->db_array;
    }


    // set the database connection
    function setConnection($db)
    {
	$this->connection = $db;
	if ($this->isConnected() === FALSE)
	{
	    error_log("Invalid database connection.");
	    error_log("Query setDB called with invalid dbconnect: ".
		      get_class($this->connection) .".");
	}
    }

    // open connection with database.
    // result is TRUE on success, FALSE on failure.
    function openConnection()
    {
	if ($this->connection != NULL)
	{
	    Debug::printDebug("Connection is OPEN!");
	    return TRUE;
	}
	else
	{
	    global $sidb;
	    //$db_host = $sidb->database->host;
	    //$db_port = $sidb->database->port;
	    //$db_name = $sidb->database->name;
	    //$db_user = $sidb->database->user;
	    //$db_pass = $sidb->database->pass;
	global $db_host, $db_port, $db_name, $db_user, $db_pass;

	    if (!isset($db_name) || $db_name == "" ||
		!isset($db_port) || $db_port == "" ||
		!isset($db_host) || $db_host == "" ||
		!isset($db_user) || $db_user == "" ||
		!isset($db_pass) )
	    {
		error_log("Invalid database connection parameters.");
		return FALSE;
	    }

	    $this->connection = @pg_connect("dbname=". $db_name ." ".
					"port=".   $db_port ." ".
					"host=".   $db_host ." ".
					"user=".   $db_user ." ".
					"password=". $db_pass );
	    if ($this->connection === FALSE)
	    {
		error_log("Can't open database connection.");
		return FALSE;
	    }
	}

	return $this->connection;
    }

    // check whether database connection is open.
    // result is TRUE on success, FALSE on failure.
    function isConnected()
    {
	if ($this->connection == NULL )
	{
	    return FALSE;
	}
	else
	{
	    return TRUE;
	}
    }

    // return DB connection handle.
    function getConnection()
    {
	return $this->connection;
    }

    // close connection with database.
    function closeConnection()
    {
	if ($this->connection != NULL)
	{
	    if (@pg_close($this->connection) !== TRUE)
	    {
		error_log("Can't close database connection.");
	    }
	    $this->connection = NULL;
	}
    }

}

?>
