<?php

// expire_inc.php - Deletes images after their expiration date
// expire_inc.php - author: Nico Stuurman
  /***************************************************************************
  *                                                                          *
  * When the expiration date is reached, a warning is sent to the image's    *
  * owner.  When the warning email was sent more than a week ago, the image  *
  * will be deleted. This code is now executed when the 'vacuum' button is   *
  * pressed.  Eventually, a stand alone version can be run as a CRON job.    *
  *                                                                          *
  * Authors: SVI group University Utrecht and Nico Stuurman                  *
  * email: stuurman@rulbim.leidenuniv.nl                                     *
  * demo at: https://impi.leidenuniv.nl                                      *
  * Created: 07/07/01                                                        *
  * Last modified 07/07/01                                                   *
  * Copyright (c) 2001 by Nico Stuurman                                      *
  * ------------------------------------------------------------------------ *
  *  This program is free software; you can redistribute it and/or modify it *
  *  under the terms of the GNU General Public License as published by the   *
  *  Free Software Foundation; either version 2 of the License, or (at your  *
  *  option) any later version.                                              *
  \**************************************************************************/


// get email addres of sysadmin
$query = "SELECT email FROM users WHERE login='sysadmin';";
$result = do_database_query ($query);
$r = db_fetch_array ($result, 0);
$from = $r["email"];

// get current Unix time stamp
$now = time ();

// find all images that are expired but where no warning has been sent
$query = "SELECT imageid,filename,exp_date,exp_mail_date,userid 
          FROM images WHERE (exp_date < $now) AND (exp_date <> 0);";
$result = do_database_query ($query);
$number_of_results = db_numrows ($result);

for ($i = 0; $i < $number_of_results; $i++) {
   $r = db_fetch_array ($result, $i);
   $imageid= $r["imageid"];
   $imagename = $r["filename"];
   $exp_date = $r["exp_date"];
   $exp_mail_date = $r["exp_mail_date"];
   $userid = $r["userid"];
   if (! $exp_mail_date) {
      
      // no mail sent yet, so do it now
      $mail_addres = get_user_info ($userid, "email");
      $user_name = get_user_info ($userid, "real_name");
      $subject = "Your image $imagename will be deleted within a week.";
      $exp_text = date ($date_format, $exp_date);
      $message = "This message has been automatically generated by SIDB.\n\n";
      $message .= "As to your instructions, the image $imagename that you "; 
      $message .= "deposited in the image database will be deleted as of ";
      $message .= "$exp_text.  If you changed your mind, please adjust the "; 
      $message .= "image's expiration date by visiting the URL: ";
      $message .= "http://$sidb_root/imagemod.php?imageid=$imageid.";
      //echo $message."<br>";
      if ($mail_addres) {
         mail ($mail_addres, $subject, $message, "From: $from\nReply-To: $from");
         echo "Sent mail anouncing removal of image: 
            <a href=\"http://$sidb_root/show.php?id=$imageid\">
            $imagename</a> belonging to $user_name to $mail_addres.<br>\n";
      }
      else
         echo "Can not send mail to <b>$user_name</b>, but I am gonna remove 
               his/her image <a href=\"http://$sidb_root/show.php?id=$imageid\">
               $imagename>/a> next week anyway.";


      // set exp_mail_date in the database
      $query = "UPDATE images SET exp_mail_date=$now WHERE imageid=$imageid;";
      do_database_query ($query);
   }

   // if a mail has been sent more than a week ago, delete the image
   elseif ($now > ($exp_mail_date +  604800) ) {
      imgdel ($imageid); 
      echo "Deleted image <b>$imagename</b>( id: $imageid), belonging to user:
            $user_name.<br>\n";
   }

}

// vacuum the database
db_exec ("VACUUM;");
db_exec ("VACUUM ANALYZE;");
echo "<h5>Cleaned the database.</h5><br>";

// compress old images
if ($bzip2 && $compress_delay) {
   // check wether we have bzip2 or gzip
   $compresser = basename($bzip2);
   if ($compresser=="bzip2")
      $extension=".bz2";
   elseif ($compresser="gzip")
      $extension=".gz";
   else {
      echo "<h5>$bzip2 can not be used for file compression.  Please select bzip2 or gzip in the 'globals' section.<br>";
      exit();
   }
   $query = "SELECT imageid,filename,date_time,compressed,filetype 
             FROM images WHERE (date_time < $now-$compress_delay) 
             AND (compressed !='t' OR compressed IS NULL) AND
             filetype != 'ICS'";
   // ICS files are compressed by default!
   //why is this here again?  do_database_query ($query);
   $result = do_database_query ($query);
   $i==0;
   while ($r=db_fetch_array($result,$i)) {   
      $compressed = $r["compressed"];
      $imageid= $r["imageid"];
      Debug::printDebug("$imageid: $compressed");
      if ($compressed!='t') {
         $imagename = $r["filename"];
         $date_time = $r["date_time"];
         $filetype = $r["filetype"];
         $location = get_image_location ($imageid);
         $locus = escapeshellarg($location);
         system("$bzip2 $locus");
         if ($filetype =="ICS"){
            $location = str_replace (".ics", ".ids", $location);
            $location = str_replace (".ICS", ".IDS", $location);
            $locus = escapeshellarg($location);
            system("$bzip2 $locus");
         }
         if (@filesize ($location.$extension)) {
            echo "Compressed $location<br>";
            $query = "UPDATE images SET compressed='t' WHERE imageid=$imageid";
            do_database_query ($query);
         }
         else 
            echo "Failed to compress $location.<br>";
      }
      $i++;
   }
}
?>
