<?php
  
// thumbnails_inc.php - functions to generate thumbnails.
// thumbnail_inc.php - author: Nico Stuurman 
  /***************************************************************************  
  * functions to generate thumbnails                                         *
  * Created 29-08-2000 by Nico                                               *
  * Functions moved outof upload.php.                                        *
  * They are now also called from within show.inc                            *
  *   ffmpeg -r 3 -an -i %04d.ppm -an -r 30 -qscale 2 out.mpg                *
  * new_thumb_name ($id, $thumb_nr)                                          *
  * get_huygens_data ($imageid, $file_name)                                  *
  * movie_string ($tiffdir, $delay, $x, $y, $movie_name)                     *
  * huygens_thumbnail($location_name, $imageid)                              *
  * other_thumbnail($location_name, $imageid)                                *
  * get_iptc_comments ($location_name, $imageid)                             *
  * import_thumbnail ($location_name, $imageid);                             *
  *                                                                          *
  * Authors: SVI group University Utrecht and Nico Stuurman                  *
  * email: stuurman@rulbim.leidenuniv.nl                                     *
  * demo at: https://impi.leidenuniv.nl                                      *
  * Last modified 12/17/00                                                   *
  * Copyright (c) 2000 by Nico Stuurman                                      *
  * ------------------------------------------------------------------------ *
  *  This program is free software; you can redistribute it and/or modify it *
  *  under the terms of the GNU General Public License as published by the   *
  *  Free Software Foundation; either version 2 of the License, or (at your  *
  *  option) any later version.                                              *
  * $Id:                                                                     *
  \**************************************************************************/  

////
// to deal with caching problems we need a new thumbname every time the thumbs image changes
function new_thumb_name ($id, &$thumb_nr) {
   if ($thumb_nr)
      if ($thumb_nr < 9999)
         $thumb_nr+=1;
      else
         $thumb_nr = 1;
   else
      $thumb_nr = 1;
   return  $id."_".$thumb_nr.".jpg";
}

////
// function that interprets huygens output and writes variables back into the database when not set before.  
function get_huygens_data ($imageid, $file_name)
{
    global $image_tool;

    // Set correct command parameters.
    $exec_huygens = "$image_tool -noTk";
    $exec_script = "-script scripts/getParameters.tcl";
    $exec_image = "-image '$file_name'";

    // Now call the script with the parameters.
    $cmd = "$exec_huygens $exec_script $exec_image";
    //print("<br>DEBUG: $cmd");
    $fd = popen("$cmd 2>&1", "r");
    if (!$fd)
    {
        print("Could not start the process to get the thumbnail parameters.");
    }
    else
    {
        // Get lines as long as output is not eof.
        while (isset($fd) && !feof($fd))
        {
            $line = fgets($fd, 4096);
            $lastline = rtrim($line);
	    //print("<br>DEBUG: $lastline");
            if ($lastline !="")
            {
                if (strstr($lastline, "ERROR"))
                {
                    print("Could not get thumbnail parameters: \"$lastline\"");
                }
		else
		{
		    // Make distiction between the two returned lines.
		    if (strstr($lastline, "OK pars"))
		    {
		        $text = strtok($lastline, "{}");
		        $samplesizes = strtok("{}");
  
		        $x = strtok($samplesizes, " ");
		        $x = (int) (1000 * $x);
		        $y = strtok(" ");
		        $y = (int) (1000 * $y);
		        $z = strtok(" ");
		        $z = (int) (1000 * $z);
		        $t = strtok(" ");
			$t = (float) $t;
			//print("<br>DEBUG needed: x:$x, y:$y, z:$z, t:$t");
		    }
		    else if(strstr($lastline, "OK dims"))
		    {
		        $text = strtok($lastline, " ");
		        $text = strtok(" ");
		        $x_pixel = strtok(" ");
		        $y_pixel = strtok(" ");
		        $z_pixel = strtok(" ");
			//print("<br>DEBUG needed: x:$x_pixel, y:$y_pixel, z:$z_pixel");
		    }
                }
            }
        }
    }
    pclose ($fd);

      // if x, y, z, t had not been set before, set them now:
      if ($x || $y || $z || $t|| $x_pixel || $y_pixel || $z_pixel) {
         // get data from datatbase, pixel size are never set by the user
         $db_query = "SELECT x_nm, y_nm, z_nm, t_sec FROM images WHERE imageid=$imageid;";
         $db_result = do_database_query ($db_query);
         if (db_numrows($db_result)==1) {
            $db_row = db_fetch_array ($db_result, 0);
            
            // need to set each parameter individually. 
            if (!($db_row["x_nm"]) && $x) {
               $x_nm = $x;
               $test = true;
            }
            else
               $x_nm = $db_row["x_nm"];
            if (!($db_row["y_nm"]) && $y) {
               $y_nm = $y;
               $test = true;
            }
            else
               $y_nm = $db_row["y_nm"];
            if (!($db_row["z_nm"]) && $z) {
               $z_nm = $z;
               $test = true;
            }
            else
               $z_nm = $db_row["z_nm"];
            if (!($db_row["t_sec"]) && $t) {
               $t_sec = $t;
               $test = true;
            }
            else
               $t_sec = $db_row["t_sec"];

            // only when something changed update the database
            if ($test || $x_pixel || $y_pixel || $z_pixel) {
               $db_query = "UPDATE images SET x_nm='$x_nm', y_nm='$y_nm', z_nm='$z_nm', t_sec='$t_sec',
                            x_pixel='$x_pixel', y_pixel='$y_pixel', z_pixel='$z_pixel' WHERE imageid=$imageid;";
               do_database_query ($db_query);
            }
         }
	 return array('x_nm'=>$x_nm,'y_nm'=>$y_nm,'z_nm'=>$z_nm,'t_sec'=>$t_sec,
	        'x_pixel'=>$x_pixel,'y_pixel'=>$y_pixel,'z_pixel'=>$z_pixel);
    } 
}


////
// generates the right command for ffmpeg to make a movie
// 30 farmes/sec/$delay = frame rate
// $x/$y are size of image in pixels
function ffmpeg_string ($tiffdir, $delay, $x, $y, $movie_name, $ppm=false) {
   global $convert_path, $movie_tool;
   // $tiffdir contains tiffs or ppms, depending on $ppm setting
   // convert tiffs to ppms in a temp directory, keep track of their number
   mkdir ("$tiffdir/tmp", 0700);

   // ffmpeg only handles multiples of 16, add a border if it is not OK
   $xb=$x%16;
   $yb=$y%16;
   if ($xb || $yb) {
      $xb=16-$xb;
      $yb=16-$yb;
      $addborder="-border $xb".x."$yb";
      $xs=$x+$xb;
      $ys=$y+$yb;
   }
   else {
      $xs=$x;
      $ys=$y;
   }
   $dirhandle = opendir ("$tiffdir");
   $j = 0;
   while ($file = readdir ($dirhandle)) {
      if ($file != "." && $file != ".." && $file != "" && $file != "tmp") {
         $j = sprintf ("%04d", $j);
         if ($ppm)
            $out_string .= "mv $tiffdir/$file $tiffdir/tmp/a$j.ppm >> /dev/null\n";
         else
            $out_string .= "$convert_path -sample $x".x."$y $addborder $tiffdir/$file ppm:$tiffdir/tmp/a$j.ppm >> /dev/null \n";
         $j++;
      }
   }
   $framerate = (int) (30/$delay);
   //$duration=15/$framerate;
   if ($framerate < 1)
      $framerate = 1;
   $out_string .= "$movie_tool -r $framerate -an -s $xs"."x"."$ys -i $tiffdir/tmp/a%04d.ppm -r 30 -qscale 2 -an -s $xs"."x"."$ys $movie_name"; 

   return $out_string;
}


////
// mpeg2enc is part of the mjpeg suite of tools
// mpeg2enc is currently the best movietool.  Fast and good.
// assume all tools to be there....
function mpeg2enc_string ($tiffdir, $delay, $x, $y, $movie_name, $ppm=false) {
   global $convert_path, $movie_tool;
   // $tiffdir contains tiffs or ppms, depending on $ppm setting
   // convert tiffs to ppms in a temp directory, keep track of their number
   mkdir ("$tiffdir/tmp", 0700);

   // ppmtoy4m only handles multiples of 2, add a border if it is not OK
   $x=$x+$x%2;
   $y=$y+$y%2;
   $dirhandle = opendir ("$tiffdir");
   // process the files, and build the desired delay
   while ($file = readdir ($dirhandle)) {
      if ($file != "." && $file != ".." && $file != "" && $file != "tmp") {
         $j = sprintf ("%04d", $j);
         if ($ppm) {
            for ($i=0; $i<$delay; $i++) 
               $filestring.=" $tiffdir/$file";
         }
         else {
            $out_string .= "$convert_path $tiffdir/$file -scale $x".x."$y! ppm:$tiffdir/a$j.ppm >> /dev/null \n";
            for ($i=0; $i<$delay; $i++) 
               $filestring.=" $tiffdir/a$j.ppm";
         }
      }
   }

   // should to test where cat is located.... 
   $out_string.= "/bin/cat ";
   $out_string.=$filestring;
   // make a decent estimate for the bitrate
   $bitrate=(int)(($x*$y)/($delay+3)/10);
   // hope that ppmtoy4m is in the same dir as mpeg2enc
   $ppmtoy4m=dirname($movie_tool)."/ppmtoy4m";
   if (!is_readable($ppmtoy4m)) {
      echo "Could not find the binary <i>ppmtoy4m</i> needed to make movies.<br>\n";
      return false;
   }
   // and instruct mpeg2enc, we Americans go for 30 frames/sec frame-rate
   $out_string .= "| $ppmtoy4m -F 30:1 | $movie_tool --format 0 --frame-rate 5 --quantisation 2 --video-bitrate $bitrate --video-buffer 2500 --verbose 0 -o $movie_name";
   //echo "$out_string.<br>";       

   return $out_string;
}


////
// generates a string to be inserted in a shell script for mpeg2encode
// converts a series of tiff files into an mpeg movie
// 30 frames/sec/$delay = frames/sec; 
// $x/$y are size in pixels
function movie_string ($tiffdir, $delay, $x, $y, $movie_name, $ppm=false) {
   global $convert_path, $movie_tool;

   // store ppms in a temp directory, keep track of their number
   mkdir ("$tiffdir/tmp", 0700);

   $dirhandle = opendir ("$tiffdir");
   $i = 0;
   while ($file = readdir ($dirhandle)) {
      if ($file != "." && $file != ".." && $file != "" && $file != "tmp") {
         $j = $delay * $i;
         $j = sprintf ("%04d", $j);
         if ($ppm)
            $out_string .= "mv $tiffdir/$file $tiffdir/tmp/a$j.ppm >> /dev/null\n";
         else
            $out_string .= "$convert_path -sample $x".x."$y $tiffdir/$file ppm:$tiffdir/tmp/a$j.ppm >> /dev/null \n";
         for ($k = 1; $k < $delay; $k++) {
            $l = sprintf ("%04d", $k + $j);
            $out_string .= "cp $tiffdir/tmp/a$j.ppm $tiffdir/tmp/a$l.ppm >> /dev/null \n";
         }
         $i++;
      }
   }
   if ($l)
      $last = (int) $l;
   else
      $last = (int) $j;
   closedir ($dirhandle);
   //echo $out_string;

   // generate mpeg2encode parameterfile
   $par_file = "$tiffdir/tmp/mpeg.par";
   $fo = fopen("$par_file", "w");
   if (!$fo) {
      echo "Could not open mpeg parameter file for writing.<br>\n";
      return false;
   }
   // $bitrate = 2304000/$delay;
   $bitrate = 4608000/$delay;

   $par_string .= "MPEG-1 Sequence generated by SIDB 
$tiffdir/tmp/a%04d    /* name of source files */
-         /* name of reconstructed images  */
-         /* name of intra quant matrix file   */ 
-         /* name of non intra quant matrix file  */
-         /* name of statistics file  */
2         /* input picture file format: 0=*.Y,*.U,*.V, 1=*.yuv, 2=*.ppm */ 
$last       /* number of frames */
0         /* number of first frame */
00:00:00:00 /* timecode of first frame */
12        /* N (# of frames in GOP) */
3         /* M (I/P frame distance) */
1         /* ISO/IEC 11172-2 stream */
0         /* 0:frame pictures, 1:field pictures */
$x       /* horizontal_size */
$y       /* vertical_size */
8         /* aspect_ratio_information 8=CCIR601 625 line, 9=CCIR601 525 line */
5         /* frame_rate_code 1=23.976, 2=24, 3=25, 4=29.97, 5=30 frames/sec. */
$bitrate /* bit_rate (bits/s) */
20        /* vbv_buffer_size (in multiples of 16 kbit) */
0         /* low_delay  */
0         /* constrained_parameters_flag */
4         /* Profile ID: Simple = 5, Main = 4, SNR = 3, Spatial = 2, High = 1 */
8         /* Level ID:   Low = 10, Main = 8, High 1440 = 6, High = 4          */
1         /* progressive_sequence */
1         /* chroma_format: 1=4:2:0, 2=4:2:2, 3=4:4:4 */
2         /* video_format: 0=comp., 1=PAL, 2=NTSC, 3=SECAM, 4=MAC, 5=unspec. */
5         /* color_primaries */
5         /* transfer_characteristics */
5         /* matrix_coefficients */
$x       /* display_horizontal_size */
$y       /* display_vertical_size */
0         /* intra_dc_precision (0: 8 bit, 1: 9 bit, 2: 10 bit, 3: 11 bit */
0         /* top_field_first */
1 1 1     /* frame_pred_frame_dct (I P B) */
0 0 0     /* concealment_motion_vectors (I P B) */
0 0 0     /* q_scale_type  (I P B) */
0 0 0     /* intra_vlc_format (I P B)*/
0 0 0     /* alternate_scan (I P B) */
0         /* repeat_first_field */
1         /* progressive_frame */
0         /* P distance between complete intra slice refresh */
0         /* rate control: r (reaction parameter) */
0         /* rate control: avg_act (initial average activity) */
0         /* rate control: Xi (initial I frame global complexity measure) */
0         /* rate control: Xp (initial P frame global complexity measure) */
0         /* rate control: Xb (initial B frame global complexity measure) */
0         /* rate control: d0i (initial I frame virtual buffer fullness) */
0         /* rate control: d0p (initial P frame virtual buffer fullness) */
0         /* rate control: d0b (initial B frame virtual buffer fullness) */
2 2 11 11 /* P:  forw_hor_f_code forw_vert_f_code search_width/height */
1 1 3  3  /* B1: forw_hor_f_code forw_vert_f_code search_width/height */
1 1 7  7  /* B1: back_hor_f_code back_vert_f_code search_width/height */
1 1 7  7  /* B2: forw_hor_f_code forw_vert_f_code search_width/height */
1 1 3  3  /* B2: back_hor_f_code back_vert_f_code search_width/height */";

   // write the script to file
   fwrite ($fo , $par_string);
   fclose($fo);

   $out_string .= "$movie_tool $par_file $movie_name >> /dev/null \n";
   
   return $out_string;

}


////
// makes an mpeg movie of a 3-D file, rotates the object.
function huygens_rotate ($imageid, $steps = 20, $total_degrees = 360, $axis = "x") {
   global $S_USERID, $image_tool, $movie_tool, $convert_path, $default_thumb_size, $max_size_for_movie;
   global $archive_dir, $thumbnail_dir, $S_OUTDIR;
   $dirname = strrchr ($S_OUTDIR, "/");

   // The movie is going to be written to the fromsid directory of the user
   if (! is_writeable ($S_OUTDIR)) {
      echo "<h3>Can not write to your directory <i>$dirname</i>.<br>\n";
      echo "Fix this first and try again.</h3>";  
      return false;
   }

   $moviename = get_image_info ($imageid, "filename");
   $dot =  strrpos ($moviename, ".");
   $moviename = substr ($moviename, 0,$dot);
   $moviename .= ".mpg";
   $movie = "$S_OUTDIR/$moviename";

   // read in desired thumb size, last used thumb nr., and filesize
   $query = "SELECT x_thumb, y_thumb, filesize FROM images WHERE imageid='$imageid';";
   $result = db_exec ($query);
   $r = db_fetch_array($result,0);
   $x_size = $r["x_thumb"];
   $y_size = $r["y_thumb"];
   $filesize = $r["filesize"];

   if (!($x_size))
      $x_size = $default_thumb_size;
   if (!($y_size))
      $y_size = $default_thumb_size;
   
   // Unique tag for files/directories created from this script
   $unique = $S_USERID . time();

   // create a script for huscript
   $huygens_script = "$archive_dir/tmp/$unique.tcl";
   $fo = fopen("$huygens_script", "w");
   if (!$fo) {
      echo "Could not open tcl file for writing.<br>\n";
      return false;
   }

   $path_file = get_image_location ($imageid, false, $location_name, $S_USERID);

   // create a directory in which the layers (to be used in making the mpeg file) can be saved
   if ($filesize <= $max_size_for_movie) {
      $huygensdir = $archive_dir ."/tmp/" .$unique;
      if (!mkdir($huygensdir, 0700)) {
         echo "Could not create huygensdir $huygensdir.<br>";
         $huygensdir = false;
      }
   }
   else
      echo "File is too big to make a movie<br>";

   // create unique filename to pipe huygens output into
   // $huygensout = $archive_dir ."/tmp/" . $S_USERID . time(). ".txt";
   // echo "huygensdir: $huygensdir, out: $huygensout.<br>"; 

   if ($huygensdir) { 
      echo "<h3>The movie <i>$moviename</i> is processing. </h3><br>\n";
      echo "Once ready it will be deposited in your directory <i>$dirname</i>.<br>\n";
 
/******************* the script instructing huygens ************************/ 

      $thumb_script .= "huOpt gundo off\n";

      // set colors for different channels 
      if ($channels = get_channels ($imageid, 1, $dummy) ) {
         for ($i=0; $i<sizeof($channels); $i++) {
            if ($channels[$i]["color"])
               $thumb_script .= "huOpt chanCol -ch $i -rgb {".get_color($channels[$i]["color"],"color_string")."}\n";
         }
      }

      $thumb_script .= 'proc saveminiMipRgb {src mip rgbmip number x_ori y_ori} {
   set name [format %04d $number]
   $src miniMIP -> $mip -dim {'.$x_size.' '. $y_size.'} 
   set xSize [$mip getdims -mode x]
   set ySize [$mip getdims -mode y]
   set borderY [expr ($y_ori - $ySize)]
   set borderX [expr ($x_ori - $xSize)]
   $mip border {$borderX $borderY}
   $mip chan2rgba -> $rgbmip -col true 
   $rgbmip save '.$huygensdir.'/$name -type ppm
}
   
set myFullPath "'.$path_file.'"
set myImage [img open $myFullPath -type byte]

set xSizeOri [$myImage getdims -mode x]
set ySizeOri [$myImage getdims -mode y]
set zSizeOri [$myImage getdims -mode z]

img create thumb -type byte
img create rgbthumb -chanspec packed -chan 4 -dim {'.$x_size.' '.$y_size.' 0} -type byte
img create temp -type byte

for {set i 0} {$i < '.$steps.' + 1} {incr i 1} {
   puts "i = $i"
   set angle [expr ($i.0 / '.$steps.'.0 * '.$total_degrees.'.0)]
   puts "angle: $angle"
   $myImage rotate -> temp -axis '.$axis.' -gamma $angle -q low 
   saveminiMipRgb temp thumb rgbthumb $i $xSizeOri $ySizeOri
}

temp del
thumb del
rgbthumb del
$myImage del
exit';

      // write the script to file
      fwrite ($fo , $thumb_script);
      fclose($fo);

      // write a temporary shell file to make movie in the background
      $temp_shell_file =  "$archive_dir/tmp/movie".time();
      $fo = fopen ($temp_shell_file,"w");
      // Huygens needs a writeable homedir.  Avoid problems by instructing bash and sh
      $script1 = "export HOME=$archive_dir/tmp/ \n";
      $script1 .= "setenv HOME $archive_dir/tmp \n";
      // all calls should redirect to /dev/null
      $script1 .= "$image_tool -script $huygens_script -noTk >> /dev/null \n";
`$script1`;
      // determine which movie tool is used
      $the_movie_tool = strrchr ($movie_tool, "/");
      $fs_size = 15;
      if ($the_movie_tool == "/convert")
         $script .= "$movie_tool $huygensdir/* mpg:$movie >> /dev/null \n";
      elseif ($the_movie_tool == "/mpeg2encode")
         $script .= movie_string ($huygensdir, $fs_size, $x_size, $y_size, $movie, true);
      elseif ($the_movie_tool == "/ffmpeg")
         $script .= ffmpeg_string ($huygensdir, $fs_size, $x_size, $y_size, $movie, true);
      elseif ($the_movie_tool == "/mpeg2enc")
         $script .= mpeg2enc_string ($huygensdir, $fs_size, $x_size, $y_size, $movie, true);
      $script .= "rm -r $huygensdir >> /dev/null \n";
      $script .= "rm $huygens_script \n";
      fwrite ($fo, $script);
      fclose ($fo);
      chmod ($temp_shell_file, 0700);
      usleep (350);
      // and run the script 
      $command = "$temp_shell_file >> /dev/null &";
      //echo "$command<br>";
      //readfile ($temp_shell_file);
      `$command`;
      //system ("$temp_shell_file >> /dev/null & ");
      // assume succes
      $movie_made = true;
      // clean up after a delay (is needed in some cases)
      usleep (100);
      unlink ($temp_shell_file);
   }
}


////
// function to create a thumbnail of uploaded file
function huygens_thumbnail($location_name, $imageid, $lateral=false)
{
    global $S_USERID;
    global $image_tool, $movie_tool, $convert_path;
    global $default_thumb_size, $max_size_for_movie;
    global $archive_dir, $thumbnail_dir;
    
    $path_file = get_image_location($imageid, false, $location_name, $S_USERID);
    $pixeldata = get_huygens_data($imageid, $path_file);
    
    // read in desired thumb size, last used thumb nr., and filesize
    $query = "SELECT x_thumb, y_thumb, fs_thumb, thumb_nr, filesize, ".
           "x_nm, y_nm, z_nm, compressed, filetype FROM images WHERE imageid='$imageid';";
    $result = db_exec ($query);
    $r = db_fetch_array($result,0);

    $xZoom = $r["x_thumb"] / $pixeldata['x_pixel'];
    $yZoom = $r["y_thumb"] / $pixeldata['y_pixel'];
    $x_nm  = $r["x_nm"];
    $y_nm  = $r["y_nm"];
    $z_nm  = $r["z_nm"];
    $filetype = $r["filetype"];

    if ($xZoom > $yZoom) {
        $Zoom= $yZoom;
    } else {
        $Zoom= $xZoom;
    }

    if ($Zoom > 3) {
        $Zoom= 3;
    }

    $myXsize = (int) ($Zoom * $pixeldata['x_pixel']);
    $myXsize = $myXsize + $myXsize%2;
    $myYsize = (int) ($Zoom * $pixeldata['y_pixel']);
    $myYsize = $myYsize + $myYsize%2;

    $thumbq = "UPDATE images SET x_thumb = ".$myXsize.", y_thumb = ".
    		$myYsize." WHERE imageid = ".$imageid.";";
    Debug::printDebug("query: $thumbq");
    db_exec ($thumbq);
 
    // make sizes even for ppmtoy4m
    #$x_size = $r["x_thumb"]+$r["x_thumb"]%2;
    #$y_size = $r["y_thumb"]+$r["y_thumb"]%2;
    $x_size = $myXsize;
    $y_size = $myYsize;
    $fs_size = $r["fs_thumb"];
    $thumb_nr = $r["thumb_nr"];
    $filesize = $r["filesize"];
 
    // if image is compressed, uncompress it
    if ($r["compressed"]=="t")
    {
	uncompress($imageid);
    }
 
    // if x is not set, take its default value
    if (!($x_size))
    {
	$x_size = $default_thumb_size;
    }

    // if y is not set, take its default value
    if (!($y_size))
    {
	$y_size = $default_thumb_size;
    }
 
    // create new thumbnail name
    $old_thumb_name = thumb_name($imageid, $thumb_nr);
    $thumb_name = new_thumb_name($imageid, $thumb_nr);
    
    // Activate the script with the correct parameters to create the thumbnail.
    $exec_huygens = "$image_tool -noTk";
    $exec_script = "-script scripts/createThumbnail.tcl";
    if ($lateral==true) {
       $exec_script = "-script scripts/createThumbnailLateral.tcl";
    }

    $extension = "." . preg_replace( "/(.*)\.(.+)$/","$2",$path_file );
    $filelink = "$archive_dir/$imageid\_link" . $extension;
    @exec ("rm $filelink"); 
    exec ("ln -s \"$path_file\" $filelink");
    Debug::printDebug("ln -s \"$path_file\" $filelink");

    if ($filetype == "ICS") {
        $pathids =get_image_location($imageid, true, $location_name, $S_USERID);
        $extension = "." . preg_replace( "/(.*)\.(.+)$/","$2",$pathids );
        if (strtolower($extension) != ".ids")
            $extension = ".ids".$extension;
        $linkids = "$archive_dir/$imageid\_link". $extension;
        @exec ("rm $linkids");
        exec ("ln -s \"$pathids\" $linkids");
        Debug::printDebug("ln -s \"$pathids\" $linkids");
    }
    $exec_image = "-image '$path_file'";
    $exec_image = "-image $filelink";
    $exec_size = "-size $x_size"."x".$y_size;
    $exec_sampling = "-sampling $x_nm"."x".$y_nm."x".$z_nm;
    $exec_output = "-output $archive_dir/tmp/thumbnail.tif";
    $exec_output .= " -output1 $archive_dir/tmp/thumbnail_1.tif";
    $exec_output .= " -output2 $archive_dir/tmp/thumbnail_2.tif";

    // set colors for different channels                       
    $exec_colors = "-chancols ";
    if ($channels = get_channels ($imageid, 1, $dummy) ) {     
       $numchan = sizeof($channels);
       $list_colors = "$numchan,";
       for ($i=0; $i<sizeof($channels); $i++) {                
         if ($channels[$i]["color"]) {
            // List colors like 255x0x0 instead of using spaces  
            $temp_color = get_color($channels[$i]["color"],"color_string");
            $temp_list_colors = split(" ",$temp_color);
            $list_colors .= implode("x",$temp_list_colors) .",";
         }
       }                                                       
       $exec_colors .= "$list_colors";
    }
    else {
       $exec_colors .= "auto";
    }

    // Now call the script with the parameters.
    $cmd = "$exec_huygens $exec_script $exec_image $exec_size $exec_output ".
            "$exec_colors $exec_sampling";
    Debug::printDebug("cmd: $cmd");
    $fd = popen("$cmd 2>&1", "r");
    if (!$fd)
    {
	print("Could not start the process to create the thumbnail.");
    }
    else
    {
	// Get lines as long as file is not closed or output is not eof.
	while (isset($fd) && !feof($fd))
	{
	    $line = fgets($fd, 1024);
	    $lastline = rtrim($line);
	    if ($lastline !="")
	    {
		if ($lastline == "OK")
		{
		    print("Thumbnail succesfully created.");
		}
		else if (strstr($lastline, "ERROR"))
		{
		    print("Thumbnail not created: \"$lastline\"");
            print(" Check for accessibility of dir $archive_dir/tmp/");
		}
	    }
	}

	// Make the thumbnail names.
	$thumbBig = "\"".$thumbnail_dir . "/big/" . $thumb_name."\"";
	$thumbBig_1 = str_replace(".jpg","_1.jpg",$thumbBig);
    $thumbBig_2 = str_replace(".jpg","_2.jpg",$thumbBig);
        $thumbSmall = "\"".$thumbnail_dir . "/small/" . $thumb_name . "\"";

	// Convert big thumbnail from tiff to jpeg.
	$cmd = "$convert_path ". #-sample $x_size"."x"."$y_size ".
	       "tiff:$archive_dir/tmp/thumbnail.tif jpeg:$thumbBig";
        $magic = exec("$cmd");

	// Convert small thumbnail from tiff to jpeg.
	$cmd = "$convert_path -geometry 70x70 ".
	       "tiff:$archive_dir/tmp/thumbnail.tif jpeg:$thumbSmall";
        $magic = exec("$cmd");

	// Remove temporary tiff thumbnail file.
        $huygensresult = exec("rm -f $archive_dir/tmp/thumbnail.tif");
        @exec ("rm $filelink"); 
        if ($filetype == "ICS") @exec ("rm $linkids"); 

    if ($lateral == true) {
	$cmd = "$convert_path ". #-sample $y_size"."x"."$y_size ".
	       "tiff:$archive_dir/tmp/thumbnail_1.tif jpeg:$thumbBig_1";
        $magic = exec("$cmd");

	$cmd = "$convert_path ". #-sample $x_size"."x"."$x_size ".
	       "tiff:$archive_dir/tmp/thumbnail_2.tif jpeg:$thumbBig_2";
        $magic = exec("$cmd");

        $huygensresult = exec("rm -f $archive_dir/tmp/thumbnail_1.tif");
        $huygensresult = exec("rm -f $archive_dir/tmp/thumbnail_2.tif");
    }

    }
    pclose($fd);




    /*
     * ????????????????? something with a movie?
     * ????????????????? in the thumbnail creation routine??
     *
    // create a directory in which the layers (to be used in making
    // the mpeg file) can be saved use the existence of the directory
    // as a flag whether or not to make the movie 13 equals No Movie
    if ($fs_size && $fs_size!=13)
    {
        if ($filesize <= $max_size_for_movie)
	{
            $huygensdir = "$archive_dir/tmp/$unique";
            if (!mkdir($huygensdir, 0700))
	    {
                echo "Could not create huygensdir $huygensdir.<br>";
                $huygensdir = false;
            }
        }
        else
        {
            echo "File is too big to make a movie<br>";
        }
    }
    */
  
 /******************* the script instructing huygens ************************/ 
 
    /*
     * We will add colors to the thumbnail script.
     * For now, we do without. We first want to get
     * the actual number of channels from the image
     * itself and not from the template.
     *
    // set colors for different channels 
    if ($channels = get_channels($imageid, 1, $dummy))
    {
	for ($i=0; $i < sizeof($channels); $i++)
	{
	    if ($channels[$i]["color"])
	    {
		$thumb_script .= "huOpt chanCol -ch $i -rgb {".
		    get_color($channels[$i]["color"],"color_string")."}\n";
	    }
	}
    }
     */

    /*
     * ????????????????? something with a movie?
     * ????????????????? in the thumbnail creation routine??
     *
   if ($huygensdir)
   { 
$thumb_script .= 'proc saveSliceRgb {src tmpImg whichPlane max} {
   $src slice -> $tmpImg -pos $whichPlane -plane xy
   $tmpImg convert -chanspec packed
   set name [format %04d $whichPlane]
   set xSize [$tmpImg getdims -mode x]
   set ySize [$tmpImg getdims -mode y]
   array set tempstats [$tmpImg stat -tclReturn]
   set tempMax $tempstats(maxValue)
   set fractionMax [expr $tempMax/$max]
   set fourChan [img create qq -chanspec packed -chan 4 -dim {$xSize $ySize 0} -type byte] 
   $tmpImg chan2rgba -> $fourChan -col true -coupleChan joint
   $fourChan * $fractionMax -> $fourChan
   $fourChan save '.$huygensdir.'/$name -type ppm
   $fourChan del
}
';
   }
     */

    /*
     * ????????????????? something with a movie?
     * ????????????????? in the thumbnail creation routine??
     *
   if ($huygensdir) {
      $thumb_script .= 'img create temp
for {set i 0} {$i < $zSizeOri} {incr i 1} {
   saveSliceRgb $myZoomedImage temp $i $max
}
temp del
';
}
     */
/***************  end of huygens script ***********************/


   /*
    * SAME MOVIE THING AGAIN
    * WILL MOVE TO SEPARATE SECTION
   $movie_array = explode (".", $thumb_name);
   $movie_name = $movie_array[0] . ".mpg";
   $movie = "\"".$thumbnail_dir . "/movie/". $movie_name ."\"";
   */


   // determine which movie tool is used
   $the_movie_tool = strrchr ($movie_tool, "/");

   // try to make a movie from tiffs produced by huygens
   if ($huygensdir && ($the_movie_tool == "/convert" || $the_movie_tool == "/mpeg2encode" || $the_movie_tool == "/ffmpeg" || $the_movie_tool == "/mpeg2enc") ) {
      // echo "A movie is being made.  Return to your images overview, ";
      // echo "and come back here in a short while.<br>";
      // echo "Please, do <b>not</b> press your browsers refresh/reload button.<br>";
      // write a temporary shell file to make movie in the background
      $temp_shell_file =  "$archive_dir/tmp/movie".time();
      $fo = fopen ($temp_shell_file,"w");
      // all calls should redirect to /dev/null
      if ($the_movie_tool == "/convert")
         $script = "$movie_tool $huygensdir/* mpg:$movie >> /dev/null \n";
      elseif ($the_movie_tool == "/mpeg2encode")
         $script = movie_string ($huygensdir, $fs_size, $x_size, $y_size, $movie, true);
      elseif ($the_movie_tool == "/ffmpeg")
         $script = ffmpeg_string ($huygensdir, $fs_size, $x_size, $y_size, $movie, true);
      elseif ($the_movie_tool == "/mpeg2enc")
         $script = mpeg2enc_string ($huygensdir, $fs_size, $x_size, $y_size, $movie, true);
      $script .= "\nrm -rf $huygensdir \n";
      fwrite ($fo, $script);
      fclose ($fo);
      chmod ($temp_shell_file, 0700);
      // and run the script 
      //system ("$temp_shell_file >> /dev/null & ");
      `"$temp_shell_file"`;
      // assume succes
      $movie_made = true;
      // clean up after a delay (is needed in some cases)
      usleep (200);
      unlink ($temp_shell_file);
   }

   // write the thumb_nr back to the database
   if ($thumb_nr) {
      if ($movie_made)
         $query = "UPDATE images SET thumb_nr='$thumb_nr', mime='video/quicktime' WHERE imageid='$imageid';";
      else
         $query = "UPDATE images SET thumb_nr='$thumb_nr' WHERE imageid='$imageid';";
      $result = db_exec ($query);
      if ($old_thumb_name) {
         $bla4 = "\"".$thumbnail_dir . "/big/" . $old_thumb_name . "\"";
         $bla5 = "\"".$thumbnail_dir . "/small/" . $old_thumb_name . "\"";
         $movie_array = explode (".", $old_thumb_name);
         $movie_name = $movie_array[0] . ".mpg";
         $bla6 = "\"".$thumbnail_dir . "/movie/". $movie_name ."\"";
         exec ("rm $bla4");
         exec ("rm $bla5");
         exec ("rm $bla6");
      }
   }
}


////
// creates thumbnails from 2D images. Calls Imagemagick's convert
function other_thumbnail ($location_name, $imageid, $action=false) {
   global $archive_dir, $thumbnail_dir, $filename_extension, $S_USERID, $convert_path;
   global $default_thumb_size, $max_size_for_convert, $movie_tool;

   // read in desired thumb size
   $query = "SELECT x_thumb,y_thumb,thumb_nr,filetype,fs_thumb,compressed FROM images WHERE imageid='$imageid';";
   $result = db_exec ($query);
   $r = db_fetch_array($result,0);
   $x_size = $r["x_thumb"];
   $y_size = $r["y_thumb"];
   $thumb_nr = $r["thumb_nr"];
   $filetype = $r["filetype"];
   $fs_size = $r["fs_thumb"];
   if ($r["compressed"]=="t")
      uncompress($imageid);

   if (!($x_size))
      $x_size = $default_thumb_size;
   if (!($y_size))
      $y_size = $default_thumb_size;

   $path_file = get_image_location ($imageid, false, $location_name, $S_USERID);
   if (filesize ($path_file) > $max_size_for_convert) {
      echo "File is too big to make a thumbnail<br>";
      return false;
   }
   $old_thumb_name = thumb_name ($imageid, $thumb_nr);
   $thumb_name = new_thumb_name ($imageid, $thumb_nr);

   // enclose string with quotes to deal with spaces and the like
   // be sure to read only the first page of a multi-page image
   $bla = "\"" . $path_file. "[0]\"";
   $bla2 = "\"".$thumbnail_dir . "/big/" . $thumb_name."\"";
   $bla3 = "\"".$thumbnail_dir . "/small/" . $thumb_name . "\"";
   
   // redirect stderr to a log file to check whether this was a 16 bit Tiff file
   $stderr_log = $archive_dir ."/tmp/" . $S_USERID . time(). ".err";

   $command = "$convert_path -verbose -colorspace RGB -geometry ".$x_size."x".$y_size." $action $bla jpg:$bla2 2> $stderr_log";
   exec($command, $result_str_arr, $status);

   // We only need to do this the first time, not when a rotation ($action) has been requested
   if (!$action) {
      // try to get this info with php native functions, if it fails, fall back to Imagemagick interpretation
      $sizearray=getimagesize($path_file);
      $pixels_x=$sizearray[0];
      $pixels_y=$sizearray[1];
      switch ($sizearray[2]) {
         case 1: $filename_extension="GIF"; break;
         case 2: $filename_extension="JPG"; break;
         case 3: $filename_extension="PNG"; break;
         case 4: $filename_extension="SWF"; break;
         case 5: $filename_extension="PSD"; break;
         case 6: $filename_extension="BMP"; break;
         case 7: $filename_extension="TIFF"; break;
         case 8: $filename_extension="TIFF"; break;
         case 9: $filename_extension="JPC"; break;
         case 10: $filename_extension="JP2"; break;
         case 11: $filename_extension="JPX"; break;
         case 12: $filename_extension="JB2"; break;
         case 13: $filename_extension="SWC"; break;
         case 14: $filename_extension="IFF"; break;
      }
      if (!$pixels_x) {
         // get filetype and size in pixels from convert. Take first token after filesize.  Don't know if it always works.
         // appparently convert yields:
         // original filename, dimensions, Class, (optional) colordepht, size (in kb), filetype, ???, ???
         $convertresult[0] = strtok ($result_str_arr[0]," ");
         $test = false;
         for ($i=1; $i<7; $i++) {
            $convertresult[$i] = strtok (" ");
            if ($i == 1) 
               $pixels = $convertresult[$i];
            if ($test) {
               $filename_extension = $convertresult[$i];
               $test = false; 
            }
            if (substr ($convertresult[$i], -2) == "kb")
               $test = true;
         }
         // extract pixel dimensions, this fails when there are spaces in the filename
         $pixels_x = (int) strtok ($pixels, "x+= >");
         $pixels_y = (int) strtok ("x+= >");
         }
      $image_insert = "UPDATE images set filetype_extension='$filename_extension', x_pixel='$pixels_x', y_pixel='$pixels_y' WHERE imageid = $imageid;";
      do_database_query($image_insert);
   } 
/*
   // 16 bit stuff seems to depend heavily on the Imagemagick version/compilation parameters
   // When this is a TIFF file, check whether it was a 16 bit file.  If so, regenerate the thumbnail.
   if (file_exists ($stderr_log)) {
      if ($filename_extension == "TIFF") 
         $bits16 = exec ("cat $stderr_log | grep 'Bits/Sample: 16'");
      unlink ($stderr_log);
   }

   // if it was a 16 bits TIFF file, just remake the thumbnail with -depth 16
   if ($bits16) {
      $command = "$convert_path -verbose -average -depth 16 -sample ".$x_size."x".$y_size." $action $bla jpg:$bla2";
      exec($command, $result_str_arr, $status);
      exec ("$convert_path -average -depth 16 -sample 70x70 $action $bla jpg:$bla3");
   }
*/      

   // make small size thumbnail for overviews

   exec ("$convert_path -colorspace RGB -geometry 70x70 $action $bla jpg:$bla3");

   // for multipage images, try making a movie
   if ( ($filetype=="multipage") && ($fs_size != 13) ) {
      $bla = "\"" . $path_file. "\"";
      $bla2 = "$archive_dir/tmp/" . $S_USERID . time();
      $movie_array = explode (".", $thumb_name);
      $movie_name = $movie_array[0] . ".mpg";
      $movie = "\"".$thumbnail_dir . "/movie/". $movie_name ."\"";
      // determine which movie tool is used
      $the_movie_tool = strrchr ($movie_tool, "/");

      // write a temporary shell file to make movie in the background
      $temp_shell_file =  "$archive_dir/tmp/movie".time();
      $fo = fopen ($temp_shell_file,"w");
      // all calls should redirect to /dev/null
      if ($the_movie_tool == "/convert")
         $script = "$movie_tool -sample ".$x_size."x".$y_size." $action $bla mpg:$movie >> /dev/null \n";
      elseif ($the_movie_tool == "/mpeg2encode") {
         if (mkdir ($bla2,0700)) {
            $command = "$convert_path +adjoin +comment -sample ".$x_size."x".$y_size." $action $bla ppm:$bla2/a";
            exec($command, $result_str_arr, $status);
            $script = movie_string 
                     ($bla2, $fs_size, $x_size, $y_size, $movie, true);
         }
      }
      elseif ($the_movie_tool == "/ffmpeg") {
         if (mkdir ($bla2,0700)) {
            // ffmpeg only handles multiples of 16, add a border if it is not OK
            $xb=$x_size%16;
            $yb=$y_size%16;
            if ($xb || $yb){
	       $xb=16-$xb;
	       $yb=16-$yb;
               $addborder="-border $xb".x."$yb";
	    }
            $command = "$convert_path +adjoin +comment $addborder -sample ".$x_size."x".$y_size." $action $bla ppm:$bla2/a";
            exec($command, $result_str_arr, $status);
            $x_size+=$xb;
            $y_size+=$yb;
            $script .= ffmpeg_string 
                     ($bla2, $fs_size, $x_size, $y_size, $movie, true);
         }
      }
      elseif ($the_movie_tool == "/mpeg2enc") {
         if (mkdir ($bla2,0700)) {
            // mpeg2enc only handles multiples of 2, add a border if it is not OK
            $x_size=$x_size+$x_size%2;
            $y_size=$y_size+$y_size%2;
            $command = "$convert_path +adjoin -scale ".$x_size."x$y_size! $action $bla ppm:$bla2/a%04d.ppm";
            exec($command, $result_str_arr, $status);
            $x_size+=$xb;
            $y_size+=$yb;
            $script .= mpeg2enc_string 
                     ($bla2, $fs_size, $x_size, $y_size, $movie, true);
         }
      }
      $script.="\nrm -rf $bla2.err\n";
      $script .= "rm -r $bla2 >> /dev/null \n";
      fwrite ($fo, $script);
      fclose ($fo);
      chmod ($temp_shell_file, 0700);
      // and run the script 
      //system ("$temp_shell_file >> /dev/null & ");
      system ("$temp_shell_file");
      // assume succes
      $movie_made = true;
      // clean up after a delay (is needed in some cases)
      usleep (200);
      unlink ($temp_shell_file);
   
      // assume succes and update the mime-type 
      $query = "UPDATE images SET thumb_nr='$thumb_nr', mime='video/quicktime' WHERE imageid='$imageid';";
      do_database_query ($query);
   }

   // write the thumb_nr back to the database
   if ($thumb_nr) {
      $query = "UPDATE images SET thumb_nr='$thumb_nr' WHERE imageid='$imageid';";
      $result = db_exec ($query);
      if ($old_thumb_name) {
         $bla4 = "\"".$thumbnail_dir . "/big/" . $old_thumb_name . "\"";
         $bla5 = "\"".$thumbnail_dir . "/small/" . $old_thumb_name . "\"";
         exec ("rm $bla4");
         exec ("rm $bla5");
         if ( ($filetype=="multipage") && ($fs_size != 13) ) {
            $movie_array = explode (".", $old_thumb_name);
            $movie_name = $movie_array[0] . ".mpg";
            $bla6 = "\"".$thumbnail_dir . "/movie/". $movie_name ."\"";
            exec ("rm $bla6");
         }
      }
   }
   
}


////
// !extracts comments outof file using functions iptcparse and exif_read_data
function get_iptc_comments ($location_name, $imageid) {
   global $S_USERID, $max_size_for_convert, $convert_path, $iptcutil_path, $archive_dir,$old_postgres;
  
$tag_spec =array (
  '2#005' => "Image Name",
  '2#5' => "Image Name",
  '2#007' => "Edit Status",
  '2#7' => "Edit Status",
  '2#010' =>"Priority",
  '2#10' =>"Priority",
  '2#015' =>"Category",
  '2#15' =>"Category",
  '2#020' =>"Supplemental Category",
  '2#20' =>"Supplemental Category",
  '2#022' =>"Fixture Identifier",
  '2#22' =>"Fixture Identifier",
  '2#025' =>"Keyword",
  '2#25' =>"Keyword",
  '2#030' =>"Release Date",
  '2#30' =>"Release Date",
  '2#035' =>"Release Time",
  '2#35' =>"Release Time",
  '2#040' =>"Special Instructions",
  '2#40' =>"Special Instructions",
  '2#045' =>"Reference Service",
  '2#45' =>"Reference Service",
  '2#047' =>"Reference Date",
  '2#47' =>"Reference Date",
  '2#050' =>"Reference Number",
  '2#50' =>"Reference Number",
  '2#055' =>"Created Date",
  '2#55' =>"Created Date",
  '2#060' =>"Created Time",
  '2#60' =>"Created Time",
  '2#065' =>"Originating Program",
  '2#65' =>"Originating Program",
  '2#070' =>"Program Version",
  '2#70' =>"Program Version",
  '2#075' =>"Object Cycle",
  '2#75' =>"Object Cycle",
  '2#080' =>"Byline",
  '2#80' =>"Byline",
  '2#085' =>"Byline Title",
  '2#85' =>"Byline Title",
  '2#090' =>"City",
  '2#90' =>"City",
  '2#095' =>"Province State",
  '2#95' =>"Province State",
  '2#100' =>"Country Code",
  '2#101' =>"Country",
  '2#103' =>"Original Transmission Reference",
  '2#105' =>"Headline",
  '2#110' =>"Credit",
  '2#115' =>"Source",
  '2#116' =>"Copyright String",
  '2#120' =>"Caption",
  '2#121' =>"Local Caption",
  '2#122' =>"Caption Writer",
  '2#200' =>"Custom Field 1",
  '2#200' =>"Custom Field 2",
  '2#200' =>"Custom Field 3",
  '2#200' =>"Custom Field 4",
  '2#200' =>"Custom Field 5",
  '2#200' =>"Custom Field 6",
  '2#200' =>"Custom Field 7",
  '2#200' =>"Custom Field 8",
  '2#200' =>"Custom Field 9",
  '2#200' =>"Custom Field 11",
  '2#200' =>"Custom Field 12",
  '2#200' =>"Custom Field 13",
  '2#200' =>"Custom Field 14",
  '2#200' =>"Custom Field 15",
  '2#200' =>"Custom Field 16",
  '2#200' =>"Custom Field 17",
  '2#200' =>"Custom Field 18",
  '2#200' =>"Custom Field 19",
  '2#200' =>"Custom Field 20",
);

   $path_file = get_image_location ($imageid, false, $location_name, $S_USERID);
   if (filesize ($path_file) > $max_size_for_convert) 
      return false;

   $description=get_image_info ($imageid, "description");

   // try reading the exif data from the file
   if (function_exists("exif_read_data")) {
      $exif=exif_read_data($path_file);
      if ($exif) {
         $excluded_exifkeys=array('FileName','FileSize','FileType','ImageWidth','ImageLength','StripOffsets','StripByteCounts');
         if ($description)
            $description .="\n";
         $description.="\n<b>Exif data:</b>\n";
         foreach ($exif as $key=>$section) {
            if (!in_array($key,$excluded_exifkeys)) {
               if (is_array($section)) {
                  foreach ($section as $name=>$val) {
                     $description.="$key.$name: $val.\n";
                  }
               }
               else
                  $description.="$key:$section.\n";
            }
         }
      }
   }
/*
   elseif ($imageinfo_path) {   // do the same with a perl utility 
      exec ( "$imageinfo_path $path_file 2>/dev/null", $iarr);
      if ($iarr) {
         $description.="$iarr";
      }
   }
*/
   elseif ($iptcutil_path) { // read the data and interpret them directly 
       // create unique filename to pipe convert output into
     
      $base=$archive_dir ."/tmp/" . $S_USERID . time(). ".iptc";
      $convertout=$base.".cvrt";
      $iptcout=$base.".iptc";
      // deal with spaces and the like (this needs to be called after file_exists!
      $path_file = '"' . $path_file . '"';
      $convert_result = `$convert_path $path_file IPTC:$convertout`;
      if (file_exists ($convertout) ) 
         `$iptcutil_path -i $convertout -o $iptcout`;
      if (file_exists($iptcout) ) {
         if ($description)
            $description .="\n";
         $description .="<b>Iptc tags:</b>\n";
         $iptcresult=file($iptcout);
         for ($i=0; $i < sizeof ($iptcresult); $i++) {
            if (ereg('^2#[0-9]{1,3}',$iptcresult[$i],$res)) {
               $key=$res[0];
               $value=substr($iptcresult[$i],strlen($key)+1);
               if (!isset($tag_spec[$key]))
                  $description.="$key: ";
               if (is_array($value)) 
                  for ($i=0;$i<sizeof($value);$i++)
                     $description.="$value[$i]";
               else
                  $description.="$value";
            }
         }
      }
      @unlink($convertout);
      @unlink($iptcout);
   }
  // $size=getimagesize($path_file,&$info);
   // older php version do not read TIFFS.  Just ignore

   if (isset($info["APP13"])){
      $iptc=iptcparse($info["APP13"]);
      if ($iptc) { 
         if ($description)
            $description .="\n";
         $description .="<b>Iptc tags:</b>\n";
         // replace iptc tags with those found in iptcutils
         while (list($key,$value)=each($iptc)){
            if ($tag_spec[$key])
               $description.=$tag_spec[$key].": ";
            else
               $description.="$key: ";
            if (is_array($value)) 
               for ($i=0;$i<sizeof($value);$i++)
                  $description.="$value[$i]\n";
            else
               $description.="$value\n";
         }
      }
   }
   

   // perform checks on $description
   $description = strip_tags ($description, "<a><i><b><hr><br><p>");
   // make sure we do not exceed the tuple size of PostgresQl (wished I could get that value somewhere dynamically
   if ($old_postgres && strlen ($description) > 8100) {
      echo "<h3>The description field became too large.  Not all image comments have been incorporated.</h3>\n";
      $description = substr ($description, 0, 8100);
   }  

   // write back to the database
  $description=ereg_replace("'","\'",$description);
   $query = "UPDATE images set description='$description' WHERE imageid = $imageid;";
   do_database_query($query);

}


////
// !retrieve metadata from a specified file. 
function get_external_metadata ($location,$image_id) 
{
   global $image_upload_directory, $old_postgres,$S_USERID;
   if (is_file($location) && is_readable($location) ) {
//echo "<!-- FOPEN $location r -->\n";
        $fh=fopen($location,"r");
        while ($line=fgets($fh,4096)){
	    $commentstring .= $line;
        }
        $description = get_image_info($image_id,"description");
        $description .= "\n".$commentstring;

        if ($old_postgres && strlen ($description) > 8100) {
            echo "<h3>The description field became too large.  Not all image comments have been incorporated.</h3>\n";
            $description = substr ($description, 0, 8100);
        }  

      // write back to the database
       $query = "UPDATE images set description='$description' WHERE imageid = $image_id;";
       do_database_query($query);
    }
}


////
// !extracts any comments that can be extracted with perls Image::Info
function get_image_comments ($image_id) 
{
    global $S_USERID, $old_postgres,$imageinfo_path,$image_upload_directory ;
    global $S_STATUS,$location_name,$upload_as_user,$G_ADMIN,$G_SUPER;

    $userid=$S_USERID;
    if (isset($upload_as_user) && $upload_as_user != $S_USERID &&($S_STATUS==$G_SUPER || $S_STATUS ==$G_ADMIN)) {
             $userid=$upload_as_user;
        }
    $imagefile=get_image_location($image_id,false,$location_name,$userid);

    exec ( "$imageinfo_path $imagefile 2>/dev/null", $iarr);
    if ($iarr) {
        $iarr[0]=""; //remove filename field
	if ($iarr[1] == "error: Unrecognized file format") { 
	   $commentstring="";
	} else {
	  $commentstring=implode($iarr,"\n");
	}
    	echo "<!-- $commentstring -->\n";
    }
    if ($commentstring){
        $description = get_image_info($image_id,"description");
        $description .= $commentstring;
	
        if ( $old_postgres && strlen ($description) > 8100) {
             echo "<h3>The description field became too large.  Not all image comments have been incorporated.</h3>\n";
             $description = substr ($description, 0, 8100);
        }  

        // write back to the database
        $query = "UPDATE images set description='$description' WHERE imageid = $image_id;";
        do_database_query($query);
    }
}


////
// function that attaches an imported thumbnail to an image 
function import_thumbnail($location, $imageid) {
   global $thumbnail_dir, $S_USERID, $convert_path;

   // final check whether the current user owns the image 
   if ($S_USERID == get_image_info ($imageid, "userid") ) {
      // check whether the file is there
      if ($location=="")
         echo "Importing a thumbnail failed.  Try selecting a file!";
      elseif ($location=="none" )
         echo "Importing a thumbnail failed.  Try selecting a file!";
      elseif (!(filesize($location)) )
         echo "No file found?";
      else {
         // find the last thumb_nr (if any)
         $query =  "SELECT thumb_nr FROM IMAGES WHERE imageid='$imageid';";
         //echo "$query<br>";
         $result = db_exec ($query);
         $result = db_exec ($query);
         if ($result) {
            $r = db_fetch_array($result,0);
            $thumb_nr = $r["thumb_nr"];
         }

         $old_thumb_name = thumb_name ($imageid, $thumb_nr);
         $thumb_name = new_thumb_name ($imageid, $thumb_nr);

         // enclose string with quotes to deal with spaces and the like
         $bla1 = "\"".$thumbnail_dir . "/big/" . $thumb_name."\"";
         $bla2 = "\"".$thumbnail_dir . "/small/" . $thumb_name . "\"";

         // make big thumb for show.php
         exec ("$convert_path -geometry 256x256 $location jpg:$bla1");
   
         // make small size thumbnail for overviews
         exec ("$convert_path -geometry 70x70 $location jpg:$bla2");
         if ($thumb_nr) {
            $query = "UPDATE images SET thumb_nr='$thumb_nr', filetype='other' WHERE imageid='$imageid';";
            $result = db_exec ($query);
            if ($old_thumb_name) {
               $bla3 = "\"".$thumbnail_dir . "/big/" . $old_thumb_name . "\"";
               $bla4 = "\"".$thumbnail_dir . "/small/" . $old_thumb_name . "\"";
               //echo "$bla4, $bla5<br>";
               exec ("rm $bla3");
               exec ("rm $bla4");
            }
         }
      } 
   }
}
   
?>
