#
# SIDB - web based Scientific Image DataBase application.
#
# Copyright (C) 2000-2003 Nico Stuurman
# Copyright (C) 2003-2004 Frank Keijzers
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: createThumbnail.tcl,v 1.6 2005/11/17 15:19:38 jose-v Exp $
#
# This file contains a special Tcl script for use with Huygens to create
# thumbnails of 3D images which are uploaded with SIDB.
#


# Just in case... catch the executed code.
if {[catch {

    # Set global undo for Huygens off.
    huOpt gundo off

    # Put all gotten arguments from the argument list into an argument array.
    array set argArray $argv

    # Get all values from arguments.
    set imageValue $argArray(-image)
    set sizeValue $argArray(-size)

    # set channel colors
    set listColors $argArray(-chancols)
    set chanColors [split $listColors ","]
    set channels [lindex $chanColors 0]

    if {$channels!="auto"} {
        for {set foo 0} {$foo<$channels} {incr foo} {
          set thisColor [split [lindex $chanColors [expr $foo+1]] "x"]
          if {$thisColor != ""} {
              huOpt chanCol -ch $foo -rgb $thisColor
          }
        }
    }


    # Set sizes.
    set sizeList [split $sizeValue "x"]
    set myXsize [lindex $sizeList 0]
    set myYsize [lindex $sizeList 1]

    # Set image filename.
    set myFullPath $imageValue
    set myImage [img open $myFullPath -type byte]

    set xSizeOri [$myImage getdims -mode x]
    set xZoom [expr $myXsize.0/$xSizeOri]

    set ySizeOri [$myImage getdims -mode y]
    set yZoom [expr $myYsize.0/$ySizeOri]


    set myZoomedImage [img create imgzoom]
    $myImage zoom -> $myZoomedImage {$xZoom $yZoom 1.0 1.0} -q standard

    img create thumb
    $myZoomedImage miniMIP -> thumb -dim {$myXsize $myYsize}
    img create rgbthumb -chanspec packed -chan 4 -dim {$myXsize $myYsize 0 0} -type byte

    thumb chan2rgba -> rgbthumb -col true
    thumb del

    # Get output location.
    set output $argArray(-output)
    rgbthumb save $output -type tiffrgb
    rgbthumb del
    $myImage del

} result]} {

    puts stderr "ERROR $result"

} else {

    puts stdout "OK"
}
exit
