#
# SIDB - web based Scientific Image DataBase application.
#
# Copyright (C) 2000-2003 Nico Stuurman
# Copyright (C) 2003-2004 Frank Keijzers
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: createThumbnailLateral.tcl,v 1.7 2006/03/27 09:39:07 jose-v Exp $
#
# This file contains a special Tcl script for use with Huygens to create
# thumbnails of 3D images which are uploaded with SIDB.
#


# Just in case... catch the executed code.
if {[catch {

    # Set global undo for Huygens off.
    huOpt gundo off
    huOpt cpu -max 2

    # Put all gotten arguments from the argument list into an argument array.
    array set argArray $argv

    # Get all values from arguments.
    set imageValue $argArray(-image)
    set sizeValue $argArray(-size)

    # set channel colors
    set listColors $argArray(-chancols)
    set chanColors [split $listColors ","]
    set channels [lindex $chanColors 0]

    if {$channels!="auto"} {
        for {set foo 0} {$foo<$channels} {incr foo} {
          set thisColor [split [lindex $chanColors [expr $foo+1]] "x"]
          if {$thisColor != ""} {
              huOpt chanCol -ch $foo -rgb $thisColor
          }
        }
    }

    set listSampling $argArray(-sampling)
    set Sampling [split $listSampling "x"]
    set xnm [expr [lindex $Sampling 0]/1000.0]
    set ynm [expr [lindex $Sampling 1]/1000.0]
    set znm [expr [lindex $Sampling 2]/1000.0]


    # Set sizes.
    set sizeList [split $sizeValue "x"]
    set myXsize [lindex $sizeList 0]
    set myYsize [lindex $sizeList 1]


    # Set image filename.
    set myFullPath $imageValue
    set myImage [img open $myFullPath]
    $myImage cs -> $myImage

    if {$xnm != 0 && $ynm != 0 && $znm != 0} {
        $myImage setp -s {$xnm $ynm $znm}
    }

    set xSizeOri [$myImage getdims -mode x]
    set ySizeOri [$myImage getdims -mode y]
    set zSizeOri [$myImage getdims -mode z]

    set ratio [expr $ySizeOri/$xSizeOri]


    set xZoom [expr $myXsize.0/$xSizeOri]
    set yZoom [expr $myYsize.0/$ySizeOri]
    set Zoom $xZoom


#    if {$xZoom > $yZoom} {
#	set Zoom $yZoom
#    } else {
#	set Zoom $xZoom
#    }

#    if {$Zoom > 2.5} {
#	#set Zoom 2.5
#    }

#    set myXsize [expr $Zoom * $xSizeOri]
#    set myYsize [expr $Zoom * $ySizeOri]
    set myZsize [expr $Zoom * $zSizeOri]

    set myZoomedImage [img create imgzoom]
    $myImage zoom -> $myZoomedImage {$Zoom $Zoom $Zoom 1.0} -q standard
    $myImage del
    $myZoomedImage cs -> $myZoomedImage
    #$myZoomedImage isosampling -> $myZoomedImage 

    img create thumb
    img create thumb1
    img create thumb2

    $myZoomedImage multiMIP -> thumb thumb2 thumb1 
    set scl [thumb2 isosampling -> thumb2]
    set scx [lindex $scl 0]
    set res [expr 1.0/$scx]
    thumb2 zoom -> thumb2 {$res $res 1 1} -q standard
    set scl [thumb1 isosampling -> thumb1]
    set scy [lindex $scl 1]
    set res [expr 1.0/$scy]
    thumb1 zoom -> thumb1 {$res $res 1 1} -q standard

    #catch {
    #pasteText "xy" thumb 1 1
    #pasteText "yz" thumb1 1 1
    #pasteText "xz" thumb2 1 1
    #} a

    img create rgbthumb -chanspec packed -chan 4 -dim {$myXsize $myYsize 0 0} -type byte
    thumb chan2rgba -> rgbthumb -col true
    thumb del
    catch { pasteText "xy" rgbthumb 1 1 }
    # Get output location.
    set output $argArray(-output)
    rgbthumb save $output -type tiffrgb
    rgbthumb del

    img create rgbthumb1 -chanspec packed -chan 4 -dim {$myZsize $myYsize 0 0} -type byte
    thumb1 chan2rgba -> rgbthumb1 -col true
    rgbthumb1 mir x
    thumb1 del
    catch { pasteText "zy" rgbthumb1 1 1 }
    set output $argArray(-output1)
    rgbthumb1 save $output -type tiffrgb
    rgbthumb1 del

    img create rgbthumb2 -chanspec packed -chan 4 -dim {$myXsize $myZsize 0 0} -type byte
    thumb2 chan2rgba -> rgbthumb2 -col true
    thumb2 del
    catch { pasteText "xz" rgbthumb2 1 1 }

    set output $argArray(-output2)
    rgbthumb2 save $output -type tiffrgb
    rgbthumb2 del

} result]} {

    puts stderr "ERROR $result"

} else {

    puts stdout "OK"
}
exit
